'''
Manage Azure API Management API Release.
'''
from .... pyaz_utils import _call_az

def list(api_id, resource_group, service_name):
    '''
    Lists all releases of an API.

    Required Parameters:
    - api_id -- API identifier. Must be unique in the current API Management service instance. Non-current revision has ;rev=n as a suffix where n is the revision number.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - service_name -- The name of the API Management service instance.
    '''
    return _call_az("az apim api release list", locals())


def show(api_id, release_id, resource_group, service_name):
    '''
    Returns the details of an API release.

    Required Parameters:
    - api_id -- API identifier. Must be unique in the current API Management service instance. Non-current revision has ;rev=n as a suffix where n is the revision number.
    - release_id -- Release identifier within an API. Must be unique in the current API Management service instance.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - service_name -- The name of the API Management service instance.
    '''
    return _call_az("az apim api release show", locals())


def create(api_id, api_revision, resource_group, service_name, if_match=None, notes=None, release_id=None):
    '''
    Creates a new Release for the API.

    Required Parameters:
    - api_id -- API identifier. Must be unique in the current API Management service instance. Non-current revision has ;rev=n as a suffix where n is the revision number.
    - api_revision -- API revision number.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - service_name -- The name of the API Management service instance.

    Optional Parameters:
    - if_match -- ETag of the Entity.
    - notes -- Release Notes.
    - release_id -- Release identifier within an API. Must be unique in the current API Management service instance.
    '''
    return _call_az("az apim api release create", locals())


def update(api_id, release_id, resource_group, service_name, add=None, api_id1=None, force_string=None, if_match=None, notes=None, remove=None, set=None):
    '''
    Updates the details of the release of the API specified by its identifier.

    Required Parameters:
    - api_id -- API identifier. Must be unique in the current API Management service instance. Non-current revision has ;rev=n as a suffix where n is the revision number.
    - release_id -- Release identifier within an API. Must be unique in the current API Management service instance.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - service_name -- The name of the API Management service instance.

    Optional Parameters:
    - add -- Add an object to a list of objects by specifying a path and key value pairs.  Example: --add property.listProperty <key=value, string or JSON string>
    - api_id1 -- Identifier of the API the release belongs to.
    - force_string -- When using 'set' or 'add', preserve string literals instead of attempting to convert to JSON.
    - if_match -- ETag of the Entity.
    - notes -- Release Notes.
    - remove -- Remove a property or an element from a list.  Example: --remove property.list <indexToRemove> OR --remove propertyToRemove
    - set -- Update an object by specifying a property path and value to set.  Example: --set property1.property2=<value>
    '''
    return _call_az("az apim api release update", locals())


def delete(api_id, release_id, resource_group, service_name, if_match=None):
    '''
    Deletes the specified release in the API.

    Required Parameters:
    - api_id -- API identifier. Must be unique in the current API Management service instance. Non-current revision has ;rev=n as a suffix where n is the revision number.
    - release_id -- Release identifier within an API. Must be unique in the current API Management service instance.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - service_name -- The name of the API Management service instance.

    Optional Parameters:
    - if_match -- ETag of the Entity.
    '''
    return _call_az("az apim api release delete", locals())




'''
Manage blob storage containers.
'''
from ... pyaz_utils import _call_az
from . import immutability_policy, lease, legal_hold, metadata, policy


def list(account_key=None, account_name=None, auth_mode=None, connection_string=None, include_deleted=None, include_metadata=None, marker=None, num_results=None, prefix=None, sas_token=None, show_next_marker=None, timeout=None):
    '''
    List containers in a storage account.

    Optional Parameters:
    - account_key -- Storage account key. Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_KEY
    - account_name -- Storage account name. Related environment variable: AZURE_STORAGE_ACCOUNT. Must be used in conjunction with either storage account key or a SAS token. If neither are present, the command will try to query the storage account key using the authenticated Azure account. If a large number of storage commands are executed the API quota may be hit
    - auth_mode -- The mode in which to run the command. "login" mode will directly use your login credentials for the authentication. The legacy "key" mode will attempt to query for an account key if no authentication parameters for the account are provided. Environment variable: AZURE_STORAGE_AUTH_MODE
    - connection_string -- Storage account connection string. Environment variable: AZURE_STORAGE_CONNECTION_STRING
    - include_deleted -- Specify that deleted containers to be returned in the response. This is for container restore enabled account. The default value is `False`
    - include_metadata -- Specify that container metadata to be returned in the response.
    - marker -- A string value that identifies the portion of the list of containers to be returned with the next listing operation. The operation returns the NextMarker value within the response body if the listing operation did not return all containers remaining to be listed with the current page. If specified, this generator will begin returning results from the point where the previous generator stopped.
    - num_results -- Specify the maximum number to return. If the request does not specify num_results, or specifies a value greater than 5000, the server will return up to 5000 items. Note that if the listing operation crosses a partition boundary, then the service will return a continuation token for retrieving the remaining of the results. Provide "*" to return all.
    - prefix -- Filter the results to return only blobs whose name begins with the specified prefix.
    - sas_token -- A Shared Access Signature (SAS). Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_SAS_TOKEN
    - show_next_marker -- Show nextMarker in result when specified.
    - timeout -- Request timeout in seconds. Applies to each call to the service.
    '''
    return _call_az("az storage container list", locals())


def delete(name, account_key=None, account_name=None, auth_mode=None, bypass_immutability_policy=None, connection_string=None, fail_not_exist=None, if_modified_since=None, if_unmodified_since=None, lease_id=None, sas_token=None, timeout=None):
    '''
    Marks the specified container for deletion.

    Required Parameters:
    - name -- The container name.

    Optional Parameters:
    - account_key -- Storage account key. Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_KEY
    - account_name -- Storage account name. Related environment variable: AZURE_STORAGE_ACCOUNT. Must be used in conjunction with either storage account key or a SAS token. If neither are present, the command will try to query the storage account key using the authenticated Azure account. If a large number of storage commands are executed the API quota may be hit
    - auth_mode -- The mode in which to run the command. "login" mode will directly use your login credentials for the authentication. The legacy "key" mode will attempt to query for an account key if no authentication parameters for the account are provided. Environment variable: AZURE_STORAGE_AUTH_MODE
    - bypass_immutability_policy -- Bypasses upcoming service behavior that will block a container from being deleted if it has a immutability-policy. Specifying this will ignore arguments aside from those used to identify the container ("--name", "--account-name").
    - connection_string -- Storage account connection string. Environment variable: AZURE_STORAGE_CONNECTION_STRING
    - fail_not_exist -- Throw an exception if the container does not exist.
    - if_modified_since -- Commence only if modified since supplied UTC datetime (Y-m-d'T'H:M'Z')
    - if_unmodified_since -- Commence only if unmodified since supplied UTC datetime (Y-m-d'T'H:M'Z')
    - lease_id -- If specified, delete_container only succeeds if the container's lease is active and matches this ID. Required if the container has an active lease.
    - sas_token -- A Shared Access Signature (SAS). Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_SAS_TOKEN
    - timeout -- Request timeout in seconds. Applies to each call to the service.
    '''
    return _call_az("az storage container delete", locals())


def show(name, account_key=None, account_name=None, auth_mode=None, connection_string=None, lease_id=None, sas_token=None, timeout=None):
    '''
    

    Required Parameters:
    - name -- The container name.

    Optional Parameters:
    - account_key -- Storage account key. Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_KEY
    - account_name -- Storage account name. Related environment variable: AZURE_STORAGE_ACCOUNT. Must be used in conjunction with either storage account key or a SAS token. If neither are present, the command will try to query the storage account key using the authenticated Azure account. If a large number of storage commands are executed the API quota may be hit
    - auth_mode -- The mode in which to run the command. "login" mode will directly use your login credentials for the authentication. The legacy "key" mode will attempt to query for an account key if no authentication parameters for the account are provided. Environment variable: AZURE_STORAGE_AUTH_MODE
    - connection_string -- Storage account connection string. Environment variable: AZURE_STORAGE_CONNECTION_STRING
    - lease_id --  If specified, get_container_properties only succeeds if the container's lease is active and matches this ID.
    - sas_token -- A Shared Access Signature (SAS). Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_SAS_TOKEN
    - timeout -- Request timeout in seconds. Applies to each call to the service.
    '''
    return _call_az("az storage container show", locals())


def create(name, account_key=None, account_name=None, auth_mode=None, connection_string=None, default_encryption_scope=None, fail_on_exist=None, metadata=None, prevent_encryption_scope_override=None, public_access=None, resource_group=None, sas_token=None, timeout=None):
    '''
    Create a container in a storage account.

    Required Parameters:
    - name -- The container name.

    Optional Parameters:
    - account_key -- Storage account key. Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_KEY
    - account_name -- Storage account name. Related environment variable: AZURE_STORAGE_ACCOUNT.
    - auth_mode -- The mode in which to run the command. "login" mode will directly use your login credentials for the authentication. The legacy "key" mode will attempt to query for an account key if no authentication parameters for the account are provided. Environment variable: AZURE_STORAGE_AUTH_MODE
    - connection_string -- Storage account connection string. Environment variable: AZURE_STORAGE_CONNECTION_STRING
    - default_encryption_scope -- Default the container to use specified encryption scope for all writes.
    - fail_on_exist -- Throw an exception if the container already exists.
    - metadata -- Metadata in space-separated key=value pairs. This overwrites any existing metadata.
    - prevent_encryption_scope_override -- Block override of encryption scope from the container default.
    - public_access -- Specifies whether data in the container may be accessed publicly.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - sas_token -- A Shared Access Signature (SAS). Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_SAS_TOKEN
    - timeout -- Request timeout in seconds. Applies to each call to the service.
    '''
    return _call_az("az storage container create", locals())


def generate_sas(name, account_key=None, account_name=None, as_user=None, auth_mode=None, cache_control=None, connection_string=None, content_disposition=None, content_encoding=None, content_language=None, content_type=None, expiry=None, https_only=None, ip=None, permissions=None, policy_name=None, start=None):
    '''
    Generate a SAS token for a storage container.

    Required Parameters:
    - name -- The container name.

    Optional Parameters:
    - account_key -- Storage account key. Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_KEY
    - account_name -- Storage account name. Related environment variable: AZURE_STORAGE_ACCOUNT. Must be used in conjunction with either storage account key or a SAS token. If neither are present, the command will try to query the storage account key using the authenticated Azure account. If a large number of storage commands are executed the API quota may be hit
    - as_user -- Indicates that this command return the SAS signed with the user delegation key. The expiry parameter and '--auth-mode login' are required if this argument is specified. 
    - auth_mode -- The mode in which to run the command. "login" mode will directly use your login credentials for the authentication. The legacy "key" mode will attempt to query for an account key if no authentication parameters for the account are provided. Environment variable: AZURE_STORAGE_AUTH_MODE
    - cache_control -- Response header value for Cache-Control when resource is accessed using this shared access signature.
    - connection_string -- Storage account connection string. Environment variable: AZURE_STORAGE_CONNECTION_STRING
    - content_disposition -- Response header value for Content-Disposition when resource is accessed using this shared access signature.
    - content_encoding -- Response header value for Content-Encoding when resource is accessed using this shared access signature.
    - content_language -- Response header value for Content-Language when resource is accessed using this shared access signature.
    - content_type -- Response header value for Content-Type when resource is accessed using this shared access signature.
    - expiry -- Specifies the UTC datetime (Y-m-d'T'H:M'Z') at which the SAS becomes invalid. Do not use if a stored access policy is referenced with --id that specifies this value.
    - https_only -- Only permit requests made with the HTTPS protocol. If omitted, requests from both the HTTP and HTTPS protocol are permitted.
    - ip -- Specifies the IP address or range of IP addresses from which to accept requests. Supports only IPv4 style addresses.
    - permissions -- The permissions the SAS grants. Allowed values: (a)dd (c)reate (d)elete (l)ist (r)ead (w)rite (a)dd (c)reate (d)elete (l)ist (r)ead (w)rite. Do not use if a stored access policy is referenced with --id that specifies this value. Can be combined.
    - policy_name -- The name of a stored access policy within the container's ACL.
    - start -- Specifies the UTC datetime (Y-m-d'T'H:M'Z') at which the SAS becomes valid. Do not use if a stored access policy is referenced with --id that specifies this value. Defaults to the time of the request.
    '''
    return _call_az("az storage container generate-sas", locals())


def exists(name, account_key=None, account_name=None, auth_mode=None, connection_string=None, sas_token=None, timeout=None):
    '''
    Check for the existence of a storage container.

    Required Parameters:
    - name -- The container name.

    Optional Parameters:
    - account_key -- Storage account key. Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_KEY
    - account_name -- Storage account name. Related environment variable: AZURE_STORAGE_ACCOUNT. Must be used in conjunction with either storage account key or a SAS token. If neither are present, the command will try to query the storage account key using the authenticated Azure account. If a large number of storage commands are executed the API quota may be hit
    - auth_mode -- The mode in which to run the command. "login" mode will directly use your login credentials for the authentication. The legacy "key" mode will attempt to query for an account key if no authentication parameters for the account are provided. Environment variable: AZURE_STORAGE_AUTH_MODE
    - connection_string -- Storage account connection string. Environment variable: AZURE_STORAGE_CONNECTION_STRING
    - sas_token -- A Shared Access Signature (SAS). Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_SAS_TOKEN
    - timeout -- Request timeout in seconds. Applies to each call to the service.
    '''
    return _call_az("az storage container exists", locals())


def set_permission(name, account_key=None, account_name=None, auth_mode=None, connection_string=None, if_modified_since=None, if_unmodified_since=None, lease_id=None, public_access=None, sas_token=None, timeout=None):
    '''
    

    Required Parameters:
    - name -- The container name.

    Optional Parameters:
    - account_key -- Storage account key. Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_KEY
    - account_name -- Storage account name. Related environment variable: AZURE_STORAGE_ACCOUNT. Must be used in conjunction with either storage account key or a SAS token. If neither are present, the command will try to query the storage account key using the authenticated Azure account. If a large number of storage commands are executed the API quota may be hit
    - auth_mode -- The mode in which to run the command. "login" mode will directly use your login credentials for the authentication. The legacy "key" mode will attempt to query for an account key if no authentication parameters for the account are provided. Environment variable: AZURE_STORAGE_AUTH_MODE
    - connection_string -- Storage account connection string. Environment variable: AZURE_STORAGE_CONNECTION_STRING
    - if_modified_since -- Commence only if modified since supplied UTC datetime (Y-m-d'T'H:M'Z')
    - if_unmodified_since -- Commence only if unmodified since supplied UTC datetime (Y-m-d'T'H:M'Z')
    - lease_id --  If specified, set_container_acl only succeeds if the container's lease is active and matches this ID.
    - public_access -- Specifies whether data in the container may be accessed publicly.
    - sas_token -- A Shared Access Signature (SAS). Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_SAS_TOKEN
    - timeout -- Request timeout in seconds. Applies to each call to the service.
    '''
    return _call_az("az storage container set-permission", locals())


def show_permission(name, account_key=None, account_name=None, auth_mode=None, connection_string=None, lease_id=None, sas_token=None, timeout=None):
    '''
    

    Required Parameters:
    - name -- The container name.

    Optional Parameters:
    - account_key -- Storage account key. Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_KEY
    - account_name -- Storage account name. Related environment variable: AZURE_STORAGE_ACCOUNT. Must be used in conjunction with either storage account key or a SAS token. If neither are present, the command will try to query the storage account key using the authenticated Azure account. If a large number of storage commands are executed the API quota may be hit
    - auth_mode -- The mode in which to run the command. "login" mode will directly use your login credentials for the authentication. The legacy "key" mode will attempt to query for an account key if no authentication parameters for the account are provided. Environment variable: AZURE_STORAGE_AUTH_MODE
    - connection_string -- Storage account connection string. Environment variable: AZURE_STORAGE_CONNECTION_STRING
    - lease_id --  If specified, get_container_acl only succeeds if the container's lease is active and matches this ID.
    - sas_token -- A Shared Access Signature (SAS). Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_SAS_TOKEN
    - timeout -- Request timeout in seconds. Applies to each call to the service.
    '''
    return _call_az("az storage container show-permission", locals())


def restore(deleted_version, name, account_key=None, account_name=None, auth_mode=None, connection_string=None, sas_token=None, timeout=None):
    '''
    Restore soft-deleted container.

    Required Parameters:
    - deleted_version -- Specify the version of the deleted container to restore.
    - name -- Specify the name of the deleted container to restore.

    Optional Parameters:
    - account_key -- Storage account key. Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_KEY
    - account_name -- Storage account name. Related environment variable: AZURE_STORAGE_ACCOUNT. Must be used in conjunction with either storage account key or a SAS token. If neither are present, the command will try to query the storage account key using the authenticated Azure account. If a large number of storage commands are executed the API quota may be hit
    - auth_mode -- The mode in which to run the command. "login" mode will directly use your login credentials for the authentication. The legacy "key" mode will attempt to query for an account key if no authentication parameters for the account are provided. Environment variable: AZURE_STORAGE_AUTH_MODE
    - connection_string -- Storage account connection string. Environment variable: AZURE_STORAGE_CONNECTION_STRING
    - sas_token -- A Shared Access Signature (SAS). Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_SAS_TOKEN
    - timeout -- Request timeout in seconds. Applies to each call to the service.
    '''
    return _call_az("az storage container restore", locals())




'''
Manage Synapse's role assignments.
'''
from .... pyaz_utils import _call_az

def create(role, workspace_name, assignee=None, assignee_object_id=None, assignee_principal_type=None, assignment_id=None, item=None, item_type=None, scope=None):
    '''
    Create a role assignment.

    Required Parameters:
    - role -- The role name/id that is assigned to the principal.
    - workspace_name -- The workspace name.

    Optional Parameters:
    - assignee -- Represent a user or service principal. Supported format: object id, user sign-in name, or service principal name.
    - assignee_object_id -- Use this parameter instead of '--assignee' to bypass Graph API invocation in case of insufficient privileges. This parameter only works with object ids for users, groups, service principals, and managed identities. For managed identities use the principal id. For service principals, use the object id and not the app id.
    - assignee_principal_type -- use with --assignee-object-id to avoid errors caused by propagation latency in AAD Graph
    - assignment_id -- Custom role assignment id in guid format, if not specified, assignment id will be randomly generated.
    - item -- Item granted access in the workspace. Using with --item-type to combine the scope of assignment
    - item_type -- Item type granted access in the workspace. Using with --item to combine the scope of assignment.
    - scope -- A scope defines the resources or artifacts that the access applies to. Synapse supports hierarchical scopes. Permissions granted at a higher-level scope are inherited by objects at a lower level. In Synapse RBAC, the top-level scope is a workspace. Assigning a role with workspace scope grants permissions to all applicable objects in the workspace.
    '''
    return _call_az("az synapse role assignment create", locals())


def list(workspace_name, assignee=None, assignee_object_id=None, item=None, item_type=None, role=None, scope=None):
    '''
    List role assignments.

    Required Parameters:
    - workspace_name -- The workspace name.

    Optional Parameters:
    - assignee -- Represent a user or service principal. Supported format: object id, user sign-in name, or service principal name.
    - assignee_object_id -- Use this parameter instead of '--assignee' to bypass Graph API invocation in case of insufficient privileges. This parameter only works with object ids for users, groups, service principals, and managed identities. For managed identities use the principal id. For service principals, use the object id and not the app id.
    - item -- Item granted access in the workspace. Using with --item-type to combine the scope of assignment
    - item_type -- Item type granted access in the workspace. Using with --item to combine the scope of assignment.
    - role -- The role name/id that is assigned to the principal.
    - scope -- A scope defines the resources or artifacts that the access applies to. Synapse supports hierarchical scopes. Permissions granted at a higher-level scope are inherited by objects at a lower level. In Synapse RBAC, the top-level scope is a workspace. Assigning a role with workspace scope grants permissions to all applicable objects in the workspace.
    '''
    return _call_az("az synapse role assignment list", locals())


def show(id, workspace_name):
    '''
    Get a role assignment by id.

    Required Parameters:
    - id -- Id of the role that is assigned to the principal.
    - workspace_name -- The workspace name.
    '''
    return _call_az("az synapse role assignment show", locals())


def delete(workspace_name, assignee=None, assignee_object_id=None, ids=None, item=None, item_type=None, role=None, scope=None, yes=None):
    '''
    Delete role assignments of workspace.

    Required Parameters:
    - workspace_name -- The workspace name.

    Optional Parameters:
    - assignee -- Represent a user or service principal. Supported format: object id, user sign-in name, or service principal name.
    - assignee_object_id -- Use this parameter instead of '--assignee' to bypass Graph API invocation in case of insufficient privileges. This parameter only works with object ids for users, groups, service principals, and managed identities. For managed identities use the principal id. For service principals, use the object id and not the app id.
    - ids -- space-separated role assignment ids. You should not provide --role or --assignee when --ids is provided.
    - item -- Item granted access in the workspace. Using with --item-type to combine the scope of assignment.Using az role assignment with filter condition before executing delete operation to be clearly aware of which assignments will be deleted.
    - item_type -- Item type granted access in the workspace. Using with --item to combine the scope of assignment.Using az role assignment with filter condition before executing delete operation to be clearly aware of which assignments will be deleted.
    - role -- The role name/id that is assigned to the principal.
    - scope -- A scope defines the resources or artifacts that the access applies to. Synapse supports hierarchical scopes. Permissions granted at a higher-level scope are inherited by objects at a lower level. In Synapse RBAC, the top-level scope is a workspace. Using az role assignment with filter condition before executing delete operation to be clearly aware of which assignments will be deleted.
    - yes -- Do not prompt for confirmation.
    '''
    return _call_az("az synapse role assignment delete", locals())




'''
Manage Azure resource group locks.
'''
from ... pyaz_utils import _call_az

def create(lock_type, name, resource_group, notes=None):
    '''
    Create a resource group lock.

    Required Parameters:
    - lock_type -- The type of lock restriction.
    - name -- Name of the lock
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - notes -- Notes about this lock.
    '''
    return _call_az("az group lock create", locals())


def delete(ids=None, name=None, resource_group=None):
    '''
    Delete a resource group lock.

    Optional Parameters:
    - ids -- One or more resource IDs (space-delimited). If provided, no other "Resource Id" arguments should be specified.
    - name -- Name of the lock
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az group lock delete", locals())


def list(filter_string=None, resource_group=None):
    '''
    List lock information in the resource-group.

    Optional Parameters:
    - filter_string -- A query filter to use to restrict the results.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az group lock list", locals())


def show(ids=None, name=None, resource_group=None):
    '''
    Show the details of a resource group lock

    Optional Parameters:
    - ids -- One or more resource IDs (space-delimited). If provided, no other "Resource Id" arguments should be specified.
    - name -- Name of the lock
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az group lock show", locals())


def update(ids=None, lock_type=None, name=None, notes=None, resource_group=None):
    '''
    Update a resource group lock.

    Optional Parameters:
    - ids -- One or more resource IDs (space-delimited). If provided, no other "Resource Id" arguments should be specified.
    - lock_type -- The type of lock restriction.
    - name -- Name of the lock
    - notes -- Notes about this lock.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az group lock update", locals())




from .... pyaz_utils import _call_az

def show(name, resource_group):
    '''
    Gets backup related properties of the Recovery Services vault.

    Required Parameters:
    - name -- Name of the Recovery services vault.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az backup vault backup-properties show", locals())


def set(name, resource_group, backup_storage_redundancy=None, cross_region_restore_flag=None, soft_delete_feature_state=None):
    '''
    Sets backup related properties of the Recovery Services vault.

    Required Parameters:
    - name -- Name of the Recovery services vault.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - backup_storage_redundancy -- Set backup storage properties for a Recovery Services vault.
    - cross_region_restore_flag -- Set cross-region-restore feature state for a Recovery Services Vault. Default: False.
    - soft_delete_feature_state -- Set soft-delete feature state for a Recovery Services Vault.
    '''
    return _call_az("az backup vault backup-properties set", locals())




'''
Manage App Configurations.
'''
from .. pyaz_utils import _call_az
from . import credential, feature, identity, kv, revision


def create(location, name, resource_group, assign_identity=None, disable_local_auth=None, enable_public_network=None, sku=None, tags=None):
    '''
    Create an App Configuration.

    Required Parameters:
    - location -- Location. Values from: `az account list-locations`. You can configure the default location using `az configure --defaults location=<location>`.
    - name -- Name of the App Configuration. You can configure the default name using `az configure --defaults app_configuration_store=<name>`
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - assign_identity -- Space-separated list of managed identities to be assigned. Use "[system]" to refer to system-assigned managed identity or a resource ID to refer to user-assigned managed identity. If this argument is provided without any value, system-assigned managed identity will be assigned by default. If this argument is not provided, no managed identities will be assigned to this App Configuration store.
    - disable_local_auth -- Disable all authentication methods other than AAD authentication.
    - enable_public_network -- When true, requests coming from public networks have permission to access this store while private endpoint is enabled. When false, only requests made through Private Links can reach this store.
    - sku -- The sku of App Configuration
    - tags -- Space-separated tags: key[=value] [key[=value] ...].
    '''
    return _call_az("az appconfig create", locals())


def delete(name, resource_group=None, yes=None):
    '''
    Delete an App Configuration.

    Required Parameters:
    - name -- Name of the App Configuration. You can configure the default name using `az configure --defaults app_configuration_store=<name>`

    Optional Parameters:
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - yes -- Do not prompt for confirmation.
    '''
    return _call_az("az appconfig delete", locals())


def update(name, disable_local_auth=None, enable_public_network=None, encryption_key_name=None, encryption_key_vault=None, encryption_key_version=None, identity_client_id=None, resource_group=None, sku=None, tags=None):
    '''
    Update an App Configuration.

    Required Parameters:
    - name -- Name of the App Configuration. You can configure the default name using `az configure --defaults app_configuration_store=<name>`

    Optional Parameters:
    - disable_local_auth -- Disable all authentication methods other than AAD authentication.
    - enable_public_network -- When true, requests coming from public networks have permission to access this store while private endpoint is enabled. When false, only requests made through Private Links can reach this store.
    - encryption_key_name -- The name of the KeyVault key.
    - encryption_key_vault -- The URI of the KeyVault.
    - encryption_key_version -- The version of the KeyVault key. Use the latest version by default.
    - identity_client_id -- Client ID of the managed identity with wrap and unwrap access to encryption key. Use system-assigned managed identity by default.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - sku -- The sku of App Configuration
    - tags -- space-separated tags: key[=value] [key[=value] ...]. Use '' to clear existing tags.
    '''
    return _call_az("az appconfig update", locals())


def list(resource_group=None):
    '''
    Lists all App Configurations under the current subscription.

    Optional Parameters:
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az appconfig list", locals())


def show(name, resource_group=None):
    '''
    Show properties of an App Configuration.

    Required Parameters:
    - name -- Name of the App Configuration. You can configure the default name using `az configure --defaults app_configuration_store=<name>`

    Optional Parameters:
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az appconfig show", locals())




from ... pyaz_utils import _call_az

def create(artifact_source_name, location, resource_group, sas_uri, artifact_root=None, tags=None):
    '''
    Creates an artifact source.

    Required Parameters:
    - artifact_source_name -- The name of the artifact source
    - location -- Location. Values from: `az account list-locations`. You can configure the default location using `az configure --defaults location=<location>`.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - sas_uri -- The SAS Uri to the Azure Storage container where the artifacts are stored.

    Optional Parameters:
    - artifact_root -- The root folder under which the artifacts are to be found. This is the path relative to the Azure storage container provided in --sas-uri.
    - tags -- space-separated tags: key[=value] [key[=value] ...]. Use '' to clear existing tags.
    '''
    return _call_az("az deploymentmanager artifact-source create", locals())


def delete(artifact_source_name, resource_group, yes=None):
    '''
    Deletes an artifact source.

    Required Parameters:
    - artifact_source_name -- The name of the artifact source
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - yes -- Do not prompt for confirmation.
    '''
    return _call_az("az deploymentmanager artifact-source delete", locals())


def show(artifact_source_name, resource_group):
    '''
    Get the details of an artifact source.

    Required Parameters:
    - artifact_source_name -- The name of the artifact source
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az deploymentmanager artifact-source show", locals())


def list(resource_group):
    '''
    List all artifact sources in a resource group.

    Required Parameters:
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az deploymentmanager artifact-source list", locals())


def update(artifact_source_name, resource_group, add=None, artifact_root=None, force_string=None, remove=None, sas_uri=None, set=None, tags=None):
    '''
    Updates an artifact source.

    Required Parameters:
    - artifact_source_name -- The name of the artifact source
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - add -- Add an object to a list of objects by specifying a path and key value pairs.  Example: --add property.listProperty <key=value, string or JSON string>
    - artifact_root -- The root folder under which the artifacts are to be found. This is the path relative to the Azure storage container provided in --sas-uri.
    - force_string -- When using 'set' or 'add', preserve string literals instead of attempting to convert to JSON.
    - remove -- Remove a property or an element from a list.  Example: --remove property.list <indexToRemove> OR --remove propertyToRemove
    - sas_uri -- The SAS Uri to the Azure Storage container where the artifacts are stored.
    - set -- Update an object by specifying a property path and value to set.  Example: --set property1.property2=<value>
    - tags -- space-separated tags: key[=value] [key[=value] ...]. Use '' to clear existing tags.
    '''
    return _call_az("az deploymentmanager artifact-source update", locals())




from ... pyaz_utils import _call_az

def list(device_name, resource_group):
    '''
    Get all the bandwidth schedules for a Data Box Edge/Data Box Gateway device.

    Required Parameters:
    - device_name -- The device name.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az databoxedge bandwidth-schedule list", locals())


def show(device_name, name, resource_group):
    '''
    Get the properties of the specified bandwidth schedule.

    Required Parameters:
    - device_name -- The device name.
    - name -- The bandwidth schedule name.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az databoxedge bandwidth-schedule show", locals())


def create(days, device_name, name, rate_in_mbps, resource_group, start, stop, no_wait=None):
    '''
    Create a bandwidth schedule.

    Required Parameters:
    - days -- The days of the week when this schedule is applicable.
    - device_name -- The device name.
    - name -- The bandwidth schedule name which needs to be added/updated.
    - rate_in_mbps -- The bandwidth rate in Mbps.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - start -- The start time of the schedule in UTC.
    - stop -- The stop time of the schedule in UTC.

    Optional Parameters:
    - no_wait -- Do not wait for the long-running operation to finish.
    '''
    return _call_az("az databoxedge bandwidth-schedule create", locals())


def update(days, device_name, name, rate_in_mbps, resource_group, start, stop, add=None, force_string=None, no_wait=None, remove=None, set=None):
    '''
    Update a bandwidth schedule.

    Required Parameters:
    - days -- The days of the week when this schedule is applicable.
    - device_name -- The device name.
    - name -- The bandwidth schedule name which needs to be added/updated.
    - rate_in_mbps -- The bandwidth rate in Mbps.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - start -- The start time of the schedule in UTC.
    - stop -- The stop time of the schedule in UTC.

    Optional Parameters:
    - add -- Add an object to a list of objects by specifying a path and key value pairs.  Example: --add property.listProperty <key=value, string or JSON string>
    - force_string -- When using 'set' or 'add', preserve string literals instead of attempting to convert to JSON.
    - no_wait -- Do not wait for the long-running operation to finish.
    - remove -- Remove a property or an element from a list.  Example: --remove property.list <indexToRemove> OR --remove propertyToRemove
    - set -- Update an object by specifying a property path and value to set.  Example: --set property1.property2=<value>
    '''
    return _call_az("az databoxedge bandwidth-schedule update", locals())


def delete(device_name, name, resource_group, no_wait=None, yes=None):
    '''
    Delete the specified bandwidth schedule.

    Required Parameters:
    - device_name -- The device name.
    - name -- The bandwidth schedule name.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - no_wait -- Do not wait for the long-running operation to finish.
    - yes -- Do not prompt for confirmation.
    '''
    return _call_az("az databoxedge bandwidth-schedule delete", locals())


def wait(device_name, name, resource_group, created=None, custom=None, deleted=None, exists=None, interval=None, timeout=None, updated=None):
    '''
    Place the CLI in a waiting state until a condition of the databoxedge bandwidth-schedule is met.

    Required Parameters:
    - device_name -- The device name.
    - name -- The bandwidth schedule name.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - created -- wait until created with 'provisioningState' at 'Succeeded'
    - custom -- Wait until the condition satisfies a custom JMESPath query. E.g. provisioningState!='InProgress', instanceView.statuses[?code=='PowerState/running']
    - deleted -- wait until deleted
    - exists -- wait until the resource exists
    - interval -- polling interval in seconds
    - timeout -- maximum wait in seconds
    - updated -- wait until updated with provisioningState at 'Succeeded'
    '''
    return _call_az("az databoxedge bandwidth-schedule wait", locals())




'''
Manage App Service Plans for an Azure Function
'''
from ... pyaz_utils import _call_az

def create(name, resource_group, sku, is_linux=None, location=None, max_burst=None, number_of_workers=None, tags=None, zone_redundant=None):
    '''
    Create an App Service Plan for an Azure Function.

    Required Parameters:
    - name -- The name of the app service plan
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - sku -- The SKU of the app service plan. e.g., F1(Free), D1(Shared), B1(Basic Small), B2(Basic Medium), B3(Basic Large), S1(Standard Small), P1V2(Premium V2 Small), PC2 (Premium Container Small), PC3 (Premium Container Medium), PC4 (Premium Container Large), I1 (Isolated Small), I2 (Isolated Medium), I3 (Isolated Large), K1 (Kubernetes).

    Optional Parameters:
    - is_linux -- host function app on Linux worker
    - location -- Location. Values from: `az account list-locations`. You can configure the default location using `az configure --defaults location=<location>`.
    - max_burst -- The maximum number of elastic workers for the plan.
    - number_of_workers -- The number of workers for the app service plan.
    - tags -- space-separated tags: key[=value] [key[=value] ...]. Use '' to clear existing tags.
    - zone_redundant -- Enable zone redundancy for high availability. Cannot be changed after plan creation. Minimum instance count is 3.
    '''
    return _call_az("az functionapp plan create", locals())


def update(name, resource_group, add=None, force_string=None, max_burst=None, number_of_workers=None, remove=None, set=None, sku=None):
    '''
    Update an App Service plan for an Azure Function.

    Required Parameters:
    - name -- The name of the app service plan
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - add -- Add an object to a list of objects by specifying a path and key value pairs.  Example: --add property.listProperty <key=value, string or JSON string>
    - force_string -- When using 'set' or 'add', preserve string literals instead of attempting to convert to JSON.
    - max_burst -- The maximum number of elastic workers for the plan.
    - number_of_workers -- The number of workers for the app service plan.
    - remove -- Remove a property or an element from a list.  Example: --remove property.list <indexToRemove> OR --remove propertyToRemove
    - set -- Update an object by specifying a property path and value to set.  Example: --set property1.property2=<value>
    - sku -- The SKU of the app service plan.
    '''
    return _call_az("az functionapp plan update", locals())


def delete(name, resource_group, yes=None):
    '''
    Delete an App Service Plan.

    Required Parameters:
    - name -- The name of the app service plan
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - yes -- Do not prompt for confirmation.
    '''
    return _call_az("az functionapp plan delete", locals())


def list(resource_group=None):
    '''
    List App Service Plans.

    Optional Parameters:
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az functionapp plan list", locals())


def show(name, resource_group):
    '''
    Get the App Service Plans for a resource group or a set of resource groups.

    Required Parameters:
    - name -- The name of the app service plan
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az functionapp plan show", locals())




"""Utility functions for the pyaz generated code to use."""

import json
import logging
import shutil
import subprocess
from typing import Dict


def _call_az(command: str, parameters: Dict) -> object:
    """
    Call an az command (supplied as a string, and parameters as dictionary).

    Calls az cli via a subprocess
    Returns the az cli json converted to python object

    Example:
    `
    _call_az("az group create", locals())
    `
    """
    # format the parameters into a list
    params = _get_params(parameters)

    # split commands into a list
    commands = command.split()

    # add the params to the commands
    commands.extend(params)

    full_command = " ".join(commands)
    logging.info(f"Executing command: {full_command}")

    # strip off az and replace it with full path to az to accomodate Windows
    commands.pop(0)
    commands.insert(0, shutil.which("az"))

    output = subprocess.run(
        commands, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE
    )
    stdout = output.stdout.decode("utf-8")
    stderr = output.stderr.decode("utf-8")
    if stdout:
        try:
            return json.loads(stdout)
        except:
            return stdout
    elif stderr:
        raise Exception(stderr)


def _get_cli_param_name(name: str) -> str:
    """
    Convert parameter name back to cli format from pythonic version.

    - Strips trailing underscore from keywords
    - Converts remaining underscores to dashes
    - Adds leading dashes
    """
    if name[-1] == "_":
        name = name[0:-1]
    name = name.replace("_", "-")
    name = f"--{name}"
    return name


def _get_params(locals: Dict) -> str:
    """
    Given the built-in locals dictionary returns a formatted string of parameters.

    The parameter string contains the az cli formatted parameter names and values
    in a comma-separated list.
    """
    # return params
    output = []

    # loop through locals and append list of parameters and their values
    # as long as the parameter has a value
    for param in locals:
        if locals[param]:

            # if value is a boolean then don't append value, just param, used for flags
            if type(locals[param]) == bool:
                output.append(_get_cli_param_name(param))
            else:
                output.append(_get_cli_param_name(param))
                output.append(locals[param])

    return output



'''
Manage helm charts for Azure Container Registries.
'''
from ... pyaz_utils import _call_az
from . import repo


def list(name, password=None, repository=None, resource_group=None, suffix=None, username=None):
    '''
    List all helm charts in an Azure Container Registry.

    Required Parameters:
    - name -- The name of the container registry. You can configure the default registry name using `az configure --defaults acr=<registry name>`

    Optional Parameters:
    - password -- The password used to log into a container registry
    - repository -- ==SUPPRESS==
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - suffix -- The tenant suffix in registry login server. You may specify '--suffix tenant' if your registry login server is in the format 'registry-tenant.azurecr.io'. Applicable if you're accessing the registry from a different subscription or you have permission to access images but not the permission to manage the registry resource.
    - username -- The username used to log into a container registry
    '''
    return _call_az("az acr helm list", locals())


def show(name, password=None, repository=None, resource_group=None, suffix=None, username=None, version=None):
    '''
    Describe a helm chart in an Azure Container Registry.

    Required Parameters:
    - name -- The name of the container registry. You can configure the default registry name using `az configure --defaults acr=<registry name>`

    Optional Parameters:
    - password -- The password used to log into a container registry
    - repository -- ==SUPPRESS==
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - suffix -- The tenant suffix in registry login server. You may specify '--suffix tenant' if your registry login server is in the format 'registry-tenant.azurecr.io'. Applicable if you're accessing the registry from a different subscription or you have permission to access images but not the permission to manage the registry resource.
    - username -- The username used to log into a container registry
    - version -- The helm chart version.
    '''
    return _call_az("az acr helm show", locals())


def delete(name, password=None, prov=None, repository=None, resource_group=None, suffix=None, username=None, version=None, yes=None):
    '''
    Delete a helm chart version in an Azure Container Registry.

    Required Parameters:
    - name -- The name of the container registry. You can configure the default registry name using `az configure --defaults acr=<registry name>`

    Optional Parameters:
    - password -- The password used to log into a container registry
    - prov -- Only delete the provenance file.
    - repository -- ==SUPPRESS==
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - suffix -- The tenant suffix in registry login server. You may specify '--suffix tenant' if your registry login server is in the format 'registry-tenant.azurecr.io'. Applicable if you're accessing the registry from a different subscription or you have permission to access images but not the permission to manage the registry resource.
    - username -- The username used to log into a container registry
    - version -- The helm chart version.
    - yes -- Do not prompt for confirmation.
    '''
    return _call_az("az acr helm delete", locals())


def push(name, force=None, password=None, repository=None, resource_group=None, suffix=None, username=None):
    '''
    Push a helm chart package to an Azure Container Registry.

    Required Parameters:
    - name -- The name of the container registry. You can configure the default registry name using `az configure --defaults acr=<registry name>`

    Optional Parameters:
    - force -- Overwrite the existing chart package.
    - password -- The password used to log into a container registry
    - repository -- ==SUPPRESS==
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - suffix -- The tenant suffix in registry login server. You may specify '--suffix tenant' if your registry login server is in the format 'registry-tenant.azurecr.io'. Applicable if you're accessing the registry from a different subscription or you have permission to access images but not the permission to manage the registry resource.
    - username -- The username used to log into a container registry
    '''
    return _call_az("az acr helm push", locals())


def install_cli(client_version=None, install_location=None, yes=None):
    '''
    Download and install Helm command-line tool.

    Optional Parameters:
    - client_version -- The target Helm CLI version. (Attention: Currently, Helm 3 does not work with "az acr helm" commands) 
    - install_location -- Path at which to install Helm CLI (Existing one at the same path will be overwritten)
    - yes -- Agree to the license of Helm, and do not prompt for confirmation.
    '''
    return _call_az("az acr helm install-cli", locals())




'''
Manage Azure Active Directory group members.
'''
from .... pyaz_utils import _call_az

def list(group):
    '''
    

    Required Parameters:
    - group -- group's object id or display name(prefix also works if there is a unique match)
    '''
    return _call_az("az ad group member list", locals())


def add(group, member_id, additional_properties=None):
    '''
    

    Required Parameters:
    - group -- group's object id or display name(prefix also works if there is a unique match)
    - member_id -- The object ID of the contact, group, user, or service principal

    Optional Parameters:
    - additional_properties -- Unmatched properties from the message are deserialized this collection
    '''
    return _call_az("az ad group member add", locals())


def remove(group, member_id):
    '''
    

    Required Parameters:
    - group -- group's object id or display name(prefix also works if there is a unique match)
    - member_id -- The object ID of the contact, group, user, or service principal
    '''
    return _call_az("az ad group member remove", locals())


def check(group, member_id):
    '''
    Check if a member is in a group.

    Required Parameters:
    - group -- group's object id or display name(prefix also works if there is a unique match)
    - member_id -- The object ID of the contact, group, user, or service principal
    '''
    return _call_az("az ad group member check", locals())




'''
Manage blob copy operations. Use `az storage blob show` to check the status of the blobs.
'''
from .... pyaz_utils import _call_az

def start(destination_blob, destination_container, account_key=None, account_name=None, auth_mode=None, connection_string=None, destination_if_match=None, destination_if_modified_since=None, destination_if_none_match=None, destination_if_unmodified_since=None, destination_lease_id=None, destination_tags_condition=None, metadata=None, rehydrate_priority=None, requires_sync=None, sas_token=None, source_account_key=None, source_account_name=None, source_blob=None, source_container=None, source_if_match=None, source_if_modified_since=None, source_if_none_match=None, source_if_unmodified_since=None, source_lease_id=None, source_path=None, source_sas=None, source_share=None, source_snapshot=None, source_tags_condition=None, source_uri=None, tags=None, tier=None, timeout=None):
    '''
    Copy a blob asynchronously. Use `az storage blob show` to check the status of the blobs.

    Required Parameters:
    - destination_blob -- Name of the destination blob. If the exists, it will be overwritten.
    - destination_container -- The container name.

    Optional Parameters:
    - account_key -- Storage account key. Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_KEY
    - account_name -- Storage account name. Related environment variable: AZURE_STORAGE_ACCOUNT. Must be used in conjunction with either storage account key or a SAS token. If neither are present, the command will try to query the storage account key using the authenticated Azure account. If a large number of storage commands are executed the API quota may be hit
    - auth_mode -- The mode in which to run the command. "login" mode will directly use your login credentials for the authentication. The legacy "key" mode will attempt to query for an account key if no authentication parameters for the account are provided. Environment variable: AZURE_STORAGE_AUTH_MODE
    - connection_string -- Storage account connection string. Environment variable: AZURE_STORAGE_CONNECTION_STRING
    - destination_if_match -- An ETag value, or the wildcard character (*). Specify this header to perform the operation only if the resource's ETag matches the value specified.
    - destination_if_modified_since -- Commence only if modified since supplied UTC datetime (Y-m-d'T'H:M'Z')
    - destination_if_none_match -- An ETag value, or the wildcard character (*). Specify this header to perform the operation only if the resource's ETag does not match the value specified. Specify the wildcard character (*) to perform the operation only if the resource does not exist, and fail the operation if it does exist.
    - destination_if_unmodified_since -- Commence only if unmodified since supplied UTC datetime (Y-m-d'T'H:M'Z')
    - destination_lease_id -- The lease ID specified for this header must match the lease ID of the estination blob. If the request does not include the lease ID or it is not valid, the operation fails with status code 412 (Precondition Failed).
    - destination_tags_condition -- Specify a SQL where clause on blob tags to operate only on blobs with a matching value.
    - metadata -- Metadata in space-separated key=value pairs. This overwrites any existing metadata.
    - rehydrate_priority -- Indicate the priority with which to rehydrate an archived blob.
    - requires_sync -- Enforce that the service will not return a response until the copy is complete.
    - sas_token -- A Shared Access Signature (SAS). Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_SAS_TOKEN
    - source_account_key -- The storage account key of the source blob.
    - source_account_name -- The storage account name of the source blob.
    - source_blob -- The blob name for the source storage account.
    - source_container -- The container name for the source storage account.
    - source_if_match -- An ETag value, or the wildcard character (*). Specify this header to perform the operation only if the resource's ETag matches the value specified.
    - source_if_modified_since -- Commence only if modified since supplied UTC datetime (Y-m-d'T'H:M'Z')
    - source_if_none_match -- An ETag value, or the wildcard character (*). Specify this header to perform the operation only if the resource's ETag does not match the value specified. Specify the wildcard character (*) to perform the operation only if the resource does not exist, and fail the operation if it does exist.
    - source_if_unmodified_since -- Commence only if unmodified since supplied UTC datetime (Y-m-d'T'H:M'Z')
    - source_lease_id -- Specify this to perform the Copy Blob operation only if the lease ID given matches the active lease ID of the source blob.
    - source_path -- The file path for the source storage account.
    - source_sas -- The shared access signature for the source storage account.
    - source_share -- The share name for the source storage account.
    - source_snapshot -- The blob snapshot for the source storage account.
    - source_tags_condition -- Specify a SQL where clause on blob tags to operate only on blobs with a matching value.
    - source_uri -- None
    - tags -- space-separated tags: key[=value] [key[=value] ...]. Use '' to clear existing tags.
    - tier -- The tier value to set the blob to. For page blob, the tier correlates to the size of the blob and number of allowed IOPS. Possible values are P10, P15, P20, P30, P4, P40, P50, P6, P60, P70, P80 and this is only applicable to page blobs on premium storage accounts; For block blob, possible values are Archive, Cool and Hot. This is only applicable to block blobs on standard storage accounts.
    - timeout -- Request timeout in seconds. Applies to each call to the service.
    '''
    return _call_az("az storage blob copy start", locals())


def cancel(copy_id, destination_blob, destination_container, account_key=None, account_name=None, auth_mode=None, connection_string=None, lease_id=None, sas_token=None, timeout=None):
    '''
    

    Required Parameters:
    - copy_id --  Copy identifier provided in the copy.id of the original copy_blob operation.
    - destination_blob -- Name of the destination blob. If the exists, it will be overwritten.
    - destination_container -- The container name.

    Optional Parameters:
    - account_key -- Storage account key. Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_KEY
    - account_name -- Storage account name. Related environment variable: AZURE_STORAGE_ACCOUNT. Must be used in conjunction with either storage account key or a SAS token. If neither are present, the command will try to query the storage account key using the authenticated Azure account. If a large number of storage commands are executed the API quota may be hit
    - auth_mode -- The mode in which to run the command. "login" mode will directly use your login credentials for the authentication. The legacy "key" mode will attempt to query for an account key if no authentication parameters for the account are provided. Environment variable: AZURE_STORAGE_AUTH_MODE
    - connection_string -- Storage account connection string. Environment variable: AZURE_STORAGE_CONNECTION_STRING
    - lease_id --  Required if the destination blob has an active infinite lease.
    - sas_token -- A Shared Access Signature (SAS). Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_SAS_TOKEN
    - timeout -- Request timeout in seconds. Applies to each call to the service.
    '''
    return _call_az("az storage blob copy cancel", locals())


def start_batch(account_key=None, account_name=None, auth_mode=None, connection_string=None, destination_container=None, destination_path=None, dryrun=None, pattern=None, sas_token=None, source_account_key=None, source_account_name=None, source_client=None, source_container=None, source_sas=None, source_share=None, source_uri=None):
    '''
    Copy multiple blobs to a blob container. Use `az storage blob show` to check the status of the blobs.

    Optional Parameters:
    - account_key -- Storage account key. Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_KEY
    - account_name -- Storage account name. Related environment variable: AZURE_STORAGE_ACCOUNT. Must be used in conjunction with either storage account key or a SAS token. If neither are present, the command will try to query the storage account key using the authenticated Azure account. If a large number of storage commands are executed the API quota may be hit
    - auth_mode -- The mode in which to run the command. "login" mode will directly use your login credentials for the authentication. The legacy "key" mode will attempt to query for an account key if no authentication parameters for the account are provided. Environment variable: AZURE_STORAGE_AUTH_MODE
    - connection_string -- Storage account connection string. Environment variable: AZURE_STORAGE_CONNECTION_STRING
    - destination_container -- The container name.
    - destination_path -- The destination path that will be prepended to the blob name.
    - dryrun -- None
    - pattern -- None
    - sas_token -- A Shared Access Signature (SAS). Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_SAS_TOKEN
    - source_account_key -- None
    - source_account_name -- None
    - source_client -- ==SUPPRESS==
    - source_container -- None
    - source_sas -- None
    - source_share -- None
    - source_uri -- None
    '''
    return _call_az("az storage blob copy start-batch", locals())




'''
Review Azure Advisor recommendations.
'''
from ... pyaz_utils import _call_az

def list(category=None, ids=None, refresh=None, resource_group=None):
    '''
    List Azure Advisor recommendations.

    Optional Parameters:
    - category -- Name of recommendation category.
    - ids -- One or more resource IDs (space-delimited). If provided, no other "Resource Id" arguments should be specified.
    - refresh -- Generate new recommendations.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az advisor recommendation list", locals())


def disable(days=None, ids=None, name=None, resource_group=None):
    '''
    Disable Azure Advisor recommendations.

    Optional Parameters:
    - days -- Number of days to disable. If not specified, the recommendation is disabled forever.
    - ids -- One or more resource IDs (space-delimited). If provided, no other "Resource Id" arguments should be specified.
    - name -- The name of the recommendation as output by the list command.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az advisor recommendation disable", locals())


def enable(ids=None, name=None, resource_group=None):
    '''
    Enable Azure Advisor recommendations.

    Optional Parameters:
    - ids -- One or more resource IDs (space-delimited). If provided, no other "Resource Id" arguments should be specified.
    - name -- The name of the recommendation as output by the list command.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az advisor recommendation enable", locals())




'''
Manage Azure Kubernetes Services.
'''
from .. pyaz_utils import _call_az
from . import command, nodepool


def browse(name, resource_group, disable_browser=None, listen_address=None, listen_port=None):
    '''
    Show the dashboard for a Kubernetes cluster in a web browser.

    Required Parameters:
    - name -- Name of the managed cluster.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - disable_browser -- None
    - listen_address -- None
    - listen_port -- None
    '''
    return _call_az("az aks browse", locals())


def create(name, resource_group, aad_admin_group_object_ids=None, aad_client_app_id=None, aad_server_app_id=None, aad_server_app_secret=None, aad_tenant_id=None, aci_subnet_name=None, admin_username=None, aks_custom_headers=None, api_server_authorized_ip_ranges=None, appgw_id=None, appgw_name=None, appgw_subnet_cidr=None, appgw_subnet_id=None, appgw_watch_namespace=None, assign_identity=None, assign_kubelet_identity=None, attach_acr=None, auto_upgrade_channel=None, client_secret=None, cluster_autoscaler_profile=None, disable_local_accounts=None, disable_public_fqdn=None, disable_rbac=None, dns_name_prefix=None, dns_service_ip=None, docker_bridge_address=None, edge_zone=None, enable_aad=None, enable_addons=None, enable_ahub=None, enable_azure_rbac=None, enable_cluster_autoscaler=None, enable_encryption_at_host=None, enable_managed_identity=None, enable_node_public_ip=None, enable_private_cluster=None, enable_rbac=None, enable_secret_rotation=None, enable_sgxquotehelper=None, enable_ultra_ssd=None, fqdn_subdomain=None, generate_ssh_keys=None, kubernetes_version=None, load_balancer_idle_timeout=None, load_balancer_managed_outbound_ip_count=None, load_balancer_outbound_ip_prefixes=None, load_balancer_outbound_ips=None, load_balancer_outbound_ports=None, load_balancer_sku=None, location=None, max_count=None, max_pods=None, min_count=None, network_plugin=None, network_policy=None, no_ssh_key=None, no_wait=None, node_count=None, node_osdisk_diskencryptionset_id=None, node_osdisk_size=None, node_osdisk_type=None, node_public_ip_prefix_id=None, node_vm_size=None, nodepool_labels=None, nodepool_name=None, nodepool_tags=None, os_sku=None, outbound_type=None, pod_cidr=None, ppg=None, private_dns_zone=None, rotation_poll_interval=None, service_cidr=None, service_principal=None, skip_subnet_role_assignment=None, ssh_key_value=None, tags=None, uptime_sla=None, vm_set_type=None, vnet_subnet_id=None, windows_admin_password=None, windows_admin_username=None, workspace_resource_id=None, yes=None, zones=None):
    '''
    Create a new managed Kubernetes cluster.

    Required Parameters:
    - name -- Name of the managed cluster.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - aad_admin_group_object_ids -- None
    - aad_client_app_id -- None
    - aad_server_app_id -- None
    - aad_server_app_secret -- None
    - aad_tenant_id -- None
    - aci_subnet_name -- None
    - admin_username -- None
    - aks_custom_headers -- None
    - api_server_authorized_ip_ranges -- None
    - appgw_id -- None
    - appgw_name -- None
    - appgw_subnet_cidr -- None
    - appgw_subnet_id -- None
    - appgw_watch_namespace -- None
    - assign_identity -- None
    - assign_kubelet_identity -- None
    - attach_acr -- None
    - auto_upgrade_channel -- None
    - client_secret -- None
    - cluster_autoscaler_profile -- Space-separated list of key=value pairs for configuring cluster autoscaler. Pass an empty string to clear the profile.
    - disable_local_accounts -- None
    - disable_public_fqdn -- None
    - disable_rbac -- None
    - dns_name_prefix -- None
    - dns_service_ip -- None
    - docker_bridge_address -- None
    - edge_zone -- The name of edge zone.
    - enable_aad -- None
    - enable_addons -- None
    - enable_ahub -- None
    - enable_azure_rbac -- None
    - enable_cluster_autoscaler -- None
    - enable_encryption_at_host -- None
    - enable_managed_identity -- None
    - enable_node_public_ip -- None
    - enable_private_cluster -- None
    - enable_rbac -- None
    - enable_secret_rotation -- None
    - enable_sgxquotehelper -- None
    - enable_ultra_ssd -- None
    - fqdn_subdomain -- None
    - generate_ssh_keys -- None
    - kubernetes_version -- None
    - load_balancer_idle_timeout -- None
    - load_balancer_managed_outbound_ip_count -- None
    - load_balancer_outbound_ip_prefixes -- None
    - load_balancer_outbound_ips -- None
    - load_balancer_outbound_ports -- None
    - load_balancer_sku -- None
    - location -- Location. Values from: `az account list-locations`. You can configure the default location using `az configure --defaults location=<location>`.
    - max_count -- None
    - max_pods -- None
    - min_count -- None
    - network_plugin -- None
    - network_policy -- None
    - no_ssh_key -- None
    - no_wait -- Do not wait for the long-running operation to finish.
    - node_count -- None
    - node_osdisk_diskencryptionset_id -- None
    - node_osdisk_size -- None
    - node_osdisk_type -- None
    - node_public_ip_prefix_id -- None
    - node_vm_size -- None
    - nodepool_labels -- space-separated labels: key[=value] [key[=value] ...]. See https://aka.ms/node-labels for syntax of labels.
    - nodepool_name -- Node pool name, up to 12 alphanumeric characters
    - nodepool_tags -- space-separated tags: key[=value] [key[=value] ...]. Use "" to clear existing tags.
    - os_sku -- None
    - outbound_type -- None
    - pod_cidr -- None
    - ppg -- None
    - private_dns_zone -- None
    - rotation_poll_interval -- None
    - service_cidr -- None
    - service_principal -- None
    - skip_subnet_role_assignment -- None
    - ssh_key_value -- None
    - tags -- space-separated tags: key[=value] [key[=value] ...]. Use '' to clear existing tags.
    - uptime_sla -- None
    - vm_set_type -- None
    - vnet_subnet_id -- None
    - windows_admin_password -- None
    - windows_admin_username -- None
    - workspace_resource_id -- None
    - yes -- Do not prompt for confirmation.
    - zones -- Space-separated list of availability zones where agent nodes will be placed.
    '''
    return _call_az("az aks create", locals())


def update(name, resource_group, aad_admin_group_object_ids=None, aad_tenant_id=None, aks_custom_headers=None, api_server_authorized_ip_ranges=None, assign_identity=None, attach_acr=None, auto_upgrade_channel=None, cluster_autoscaler_profile=None, detach_acr=None, disable_ahub=None, disable_azure_rbac=None, disable_cluster_autoscaler=None, disable_local_accounts=None, disable_public_fqdn=None, disable_secret_rotation=None, enable_aad=None, enable_ahub=None, enable_azure_rbac=None, enable_cluster_autoscaler=None, enable_local_accounts=None, enable_managed_identity=None, enable_public_fqdn=None, enable_secret_rotation=None, load_balancer_idle_timeout=None, load_balancer_managed_outbound_ip_count=None, load_balancer_outbound_ip_prefixes=None, load_balancer_outbound_ips=None, load_balancer_outbound_ports=None, max_count=None, min_count=None, no_uptime_sla=None, no_wait=None, nodepool_labels=None, rotation_poll_interval=None, tags=None, update_cluster_autoscaler=None, uptime_sla=None, windows_admin_password=None, yes=None):
    '''
    Update a managed Kubernetes cluster.

    Required Parameters:
    - name -- Name of the managed cluster.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - aad_admin_group_object_ids -- None
    - aad_tenant_id -- None
    - aks_custom_headers -- None
    - api_server_authorized_ip_ranges -- None
    - assign_identity -- None
    - attach_acr -- None
    - auto_upgrade_channel -- None
    - cluster_autoscaler_profile -- Space-separated list of key=value pairs for configuring cluster autoscaler. Pass an empty string to clear the profile.
    - detach_acr -- None
    - disable_ahub -- None
    - disable_azure_rbac -- None
    - disable_cluster_autoscaler -- None
    - disable_local_accounts -- None
    - disable_public_fqdn -- None
    - disable_secret_rotation -- None
    - enable_aad -- None
    - enable_ahub -- None
    - enable_azure_rbac -- None
    - enable_cluster_autoscaler -- None
    - enable_local_accounts -- None
    - enable_managed_identity -- None
    - enable_public_fqdn -- None
    - enable_secret_rotation -- None
    - load_balancer_idle_timeout -- None
    - load_balancer_managed_outbound_ip_count -- None
    - load_balancer_outbound_ip_prefixes -- None
    - load_balancer_outbound_ips -- None
    - load_balancer_outbound_ports -- None
    - max_count -- None
    - min_count -- None
    - no_uptime_sla -- None
    - no_wait -- Do not wait for the long-running operation to finish.
    - nodepool_labels -- space-separated labels: key[=value] [key[=value] ...]. See https://aka.ms/node-labels for syntax of labels.
    - rotation_poll_interval -- None
    - tags -- space-separated tags: key[=value] [key[=value] ...]. Use '' to clear existing tags.
    - update_cluster_autoscaler -- None
    - uptime_sla -- None
    - windows_admin_password -- None
    - yes -- Do not prompt for confirmation.
    '''
    return _call_az("az aks update", locals())


def delete(name, resource_group, no_wait=None, yes=None):
    '''
    Delete a managed Kubernetes cluster.

    Required Parameters:
    - name -- Name of the managed cluster.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - no_wait -- Do not wait for the long-running operation to finish.
    - yes -- Do not prompt for confirmation.
    '''
    return _call_az("az aks delete", locals())


def update_credentials(name, resource_group, aad_client_app_id=None, aad_server_app_id=None, aad_server_app_secret=None, aad_tenant_id=None, client_secret=None, no_wait=None, reset_aad=None, reset_service_principal=None, service_principal=None):
    '''
    Update credentials for a managed Kubernetes cluster, like service principal.

    Required Parameters:
    - name -- Name of the managed cluster.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - aad_client_app_id -- None
    - aad_server_app_id -- None
    - aad_server_app_secret -- None
    - aad_tenant_id -- None
    - client_secret -- None
    - no_wait -- Do not wait for the long-running operation to finish.
    - reset_aad -- None
    - reset_service_principal -- None
    - service_principal -- None
    '''
    return _call_az("az aks update-credentials", locals())


def disable_addons(addons, name, resource_group, no_wait=None):
    '''
    Disable Kubernetes addons.

    Required Parameters:
    - addons -- None
    - name -- Name of the managed cluster.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - no_wait -- Do not wait for the long-running operation to finish.
    '''
    return _call_az("az aks disable-addons", locals())


def enable_addons(addons, name, resource_group, appgw_id=None, appgw_name=None, appgw_subnet_cidr=None, appgw_subnet_id=None, appgw_watch_namespace=None, enable_secret_rotation=None, enable_sgxquotehelper=None, no_wait=None, rotation_poll_interval=None, subnet_name=None, workspace_resource_id=None):
    '''
    Enable Kubernetes addons.

    Required Parameters:
    - addons -- None
    - name -- Name of the managed cluster.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - appgw_id -- None
    - appgw_name -- None
    - appgw_subnet_cidr -- None
    - appgw_subnet_id -- None
    - appgw_watch_namespace -- None
    - enable_secret_rotation -- None
    - enable_sgxquotehelper -- None
    - no_wait -- Do not wait for the long-running operation to finish.
    - rotation_poll_interval -- None
    - subnet_name -- Name of an existing subnet to use with the virtual-node add-on.
    - workspace_resource_id -- None
    '''
    return _call_az("az aks enable-addons", locals())


def get_credentials(name, resource_group, admin=None, context=None, file=None, overwrite_existing=None, public_fqdn=None):
    '''
    Get access credentials for a managed Kubernetes cluster.

    Required Parameters:
    - name -- Name of the managed cluster.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - admin -- None
    - context -- If specified, overwrite the default context name.
    - file -- None
    - overwrite_existing -- None
    - public_fqdn -- None
    '''
    return _call_az("az aks get-credentials", locals())


def check_acr(acr, name, resource_group):
    '''
    Validate an ACR is accessible from an AKS cluster.

    Required Parameters:
    - acr -- None
    - name -- Name of the managed cluster.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az aks check-acr", locals())


def get_upgrades(name, resource_group):
    '''
    Get the upgrade versions available for a managed Kubernetes cluster.

    Required Parameters:
    - name -- Name of the managed cluster.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az aks get-upgrades", locals())


def install_cli(base_src_url=None, client_version=None, install_location=None, kubelogin_base_src_url=None, kubelogin_install_location=None, kubelogin_version=None):
    '''
    Download and install kubectl, the Kubernetes command-line tool. Download and install kubelogin, a client-go credential (exec) plugin implementing azure authentication.

    Optional Parameters:
    - base_src_url -- Base download source URL for kubectl releases.
    - client_version -- Version of kubectl to install.
    - install_location -- Path at which to install kubectl.
    - kubelogin_base_src_url -- Base download source URL for kubelogin releases.
    - kubelogin_install_location -- Path at which to install kubelogin.
    - kubelogin_version -- Version of kubelogin to install.
    '''
    return _call_az("az aks install-cli", locals())


def list(resource_group=None):
    '''
    List managed Kubernetes clusters.

    Optional Parameters:
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az aks list", locals())


def remove_dev_spaces(name, resource_group, yes=None):
    '''
    Remove Azure Dev Spaces from a managed Kubernetes cluster.

    Required Parameters:
    - name -- Name of the managed cluster.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - yes -- Do not prompt for confirmation
    '''
    return _call_az("az aks remove-dev-spaces", locals())


def scale(name, node_count, resource_group, no_wait=None, nodepool_name=None):
    '''
    Scale the node pool in a managed Kubernetes cluster.

    Required Parameters:
    - name -- Name of the managed cluster.
    - node_count -- None
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - no_wait -- Do not wait for the long-running operation to finish.
    - nodepool_name -- Node pool name, up to 12 alphanumeric characters
    '''
    return _call_az("az aks scale", locals())


def show(name, resource_group):
    '''
    Show the details for a managed Kubernetes cluster.

    Required Parameters:
    - name -- Name of the managed cluster.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az aks show", locals())


def upgrade(name, resource_group, control_plane_only=None, kubernetes_version=None, no_wait=None, node_image_only=None, yes=None):
    '''
    Upgrade a managed Kubernetes cluster to a newer version.

    Required Parameters:
    - name -- Name of the managed cluster.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - control_plane_only -- None
    - kubernetes_version -- None
    - no_wait -- Do not wait for the long-running operation to finish.
    - node_image_only -- None
    - yes -- Do not prompt for confirmation.
    '''
    return _call_az("az aks upgrade", locals())


def use_dev_spaces(name, resource_group, endpoint=None, space=None, update=None, yes=None):
    '''
    Use Azure Dev Spaces with a managed Kubernetes cluster.

    Required Parameters:
    - name -- Name of the managed cluster.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - endpoint -- The endpoint type to be used for a Azure Dev Spaces controller.     See https://aka.ms/azds-networking for more information.
    - space -- Name of the new or existing dev space to select. Defaults to an     interactive selection experience.
    - update -- Update to the latest Azure Dev Spaces client components.
    - yes -- Do not prompt for confirmation. Requires --space.
    '''
    return _call_az("az aks use-dev-spaces", locals())


def rotate_certs(name, resource_group, no_wait=None, yes=None):
    '''
    Rotate certificates and keys on a managed Kubernetes cluster

    Required Parameters:
    - name -- Name of the managed cluster.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - no_wait -- Do not wait for the long-running operation to finish.
    - yes -- Do not prompt for confirmation.
    '''
    return _call_az("az aks rotate-certs", locals())


def wait(name, resource_group, created=None, custom=None, deleted=None, exists=None, interval=None, timeout=None, updated=None):
    '''
    Wait for a managed Kubernetes cluster to reach a desired state.

    Required Parameters:
    - name -- Name of the managed cluster.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - created -- wait until created with 'provisioningState' at 'Succeeded'
    - custom -- Wait until the condition satisfies a custom JMESPath query. E.g. provisioningState!='InProgress', instanceView.statuses[?code=='PowerState/running']
    - deleted -- wait until deleted
    - exists -- wait until the resource exists
    - interval -- polling interval in seconds
    - timeout -- maximum wait in seconds
    - updated -- wait until updated with provisioningState at 'Succeeded'
    '''
    return _call_az("az aks wait", locals())


def stop(name, resource_group, no_wait=None):
    '''
    

    Required Parameters:
    - name -- Name of the managed cluster.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - no_wait -- Do not wait for the long-running operation to finish.
    '''
    return _call_az("az aks stop", locals())


def start(name, resource_group, no_wait=None):
    '''
    

    Required Parameters:
    - name -- Name of the managed cluster.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - no_wait -- Do not wait for the long-running operation to finish.
    '''
    return _call_az("az aks start", locals())


def get_versions(location):
    '''
    Get the versions available for creating a managed Kubernetes cluster.

    Required Parameters:
    - location -- Location. Values from: `az account list-locations`. You can configure the default location using `az configure --defaults location=<location>`.
    '''
    return _call_az("az aks get-versions", locals())




'''
Manage Azure Cosmos DB location properties.
'''
from ... pyaz_utils import _call_az

def show(location):
    '''
    Show the Azure CosmosDB location properties in the given location.

    Required Parameters:
    - location -- Name of the location
    '''
    return _call_az("az cosmosdb locations show", locals())


def list():
    '''
    Retrieves the list of cosmosdb locations and their properties.
    '''
    return _call_az("az cosmosdb locations list", locals())




'''
Manage shared access keys of a partner namespace.
'''
from ..... pyaz_utils import _call_az

def list(partner_namespace_name, resource_group):
    '''
    List shared access keys of a partner namespace.

    Required Parameters:
    - partner_namespace_name -- Name of the partner namespace
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az eventgrid partner namespace key list", locals())


def regenerate(key_name, partner_namespace_name, resource_group):
    '''
    Regenerate a shared access key of a partner namespace.

    Required Parameters:
    - key_name -- Key name to regenerate key1 or key2
    - partner_namespace_name -- Name of the partner namespace
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az eventgrid partner namespace key regenerate", locals())




'''
Manage Azure CLI configuration.
'''
from .. pyaz_utils import _call_az
from . import param_persist


def set(local=None):
    '''
    Set a configuration.

    Optional Parameters:
    - local -- Set as a local configuration in the working directory.
    '''
    return _call_az("az config set", locals())


def get(local=None):
    '''
    Get a configuration.

    Optional Parameters:
    - local -- Include local configuration. Scan from the working directory up to the root drive, then the global configuration and return the first occurrence.
    '''
    return _call_az("az config get", locals())


def unset(local=None):
    '''
    Unset a configuration.

    Optional Parameters:
    - local -- Include local configuration. Scan from the working directory up to the root drive, then the global configuration and unset the first occurrence.
    '''
    return _call_az("az config unset", locals())




from .... pyaz_utils import _call_az
from . import data_export, linked_service, linked_storage, pack, saved_search, table


def create(resource_group, workspace_name, capacity_reservation_level=None, ingestion_access=None, location=None, no_wait=None, query_access=None, quota=None, retention_time=None, sku=None, tags=None):
    '''
    Create a workspace instance

    Required Parameters:
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - workspace_name -- Name of the Log Analytics Workspace.

    Optional Parameters:
    - capacity_reservation_level -- The capacity reservation level for this workspace, when CapacityReservation sku is selected. The maximum value is 1000 and must be in multiples of 100. If you want to increase the limit, please contact LAIngestionRate@microsoft.com.
    - ingestion_access -- The public network access type to access workspace ingestion.
    - location -- Location. Values from: `az account list-locations`. You can configure the default location using `az configure --defaults location=<location>`.
    - no_wait -- Do not wait for the long-running operation to finish.
    - query_access -- The public network access type to access workspace query.
    - quota -- The workspace daily quota for ingestion in gigabytes. The minimum value is 0.023 and default is -1 which means unlimited.
    - retention_time -- The workspace data retention in days.
    - sku -- The supported value: PerGB2018, CapacityReservation.
    - tags -- space-separated tags: key[=value] [key[=value] ...]. Use '' to clear existing tags.
    '''
    return _call_az("az monitor log-analytics workspace create", locals())


def update(resource_group, workspace_name, add=None, capacity_reservation_level=None, force_string=None, ingestion_access=None, query_access=None, quota=None, remove=None, retention_time=None, set=None, tags=None):
    '''
    Update a workspace instance

    Required Parameters:
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - workspace_name -- Name of the Log Analytics Workspace.

    Optional Parameters:
    - add -- Add an object to a list of objects by specifying a path and key value pairs.  Example: --add property.listProperty <key=value, string or JSON string>
    - capacity_reservation_level -- The capacity reservation level for this workspace, when CapacityReservation sku is selected. The maximum value is 1000 and must be in multiples of 100. If you want to increase the limit, please contact LAIngestionRate@microsoft.com.
    - force_string -- When using 'set' or 'add', preserve string literals instead of attempting to convert to JSON.
    - ingestion_access -- The public network access type to access workspace ingestion.
    - query_access -- The public network access type to access workspace query.
    - quota -- The workspace daily quota for ingestion in gigabytes. The minimum value is 0.023 and default is -1 which means unlimited.
    - remove -- Remove a property or an element from a list.  Example: --remove property.list <indexToRemove> OR --remove propertyToRemove
    - retention_time -- The workspace data retention in days.
    - set -- Update an object by specifying a property path and value to set.  Example: --set property1.property2=<value>
    - tags -- space-separated tags: key[=value] [key[=value] ...]. Use '' to clear existing tags.
    '''
    return _call_az("az monitor log-analytics workspace update", locals())


def show(resource_group, workspace_name):
    '''
    Show a workspace instance.

    Required Parameters:
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - workspace_name -- Name of the Log Analytics Workspace.
    '''
    return _call_az("az monitor log-analytics workspace show", locals())


def delete(resource_group, workspace_name, force=None, yes=None):
    '''
    Delete a workspace instance.

    Required Parameters:
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - workspace_name -- Name of the Log Analytics Workspace.

    Optional Parameters:
    - force -- Deletes the workspace without the recovery option. A workspace that was deleted with this flag cannot be recovered.
    - yes -- Do not prompt for confirmation.
    '''
    return _call_az("az monitor log-analytics workspace delete", locals())


def list(resource_group=None):
    '''
    Get a list of workspaces under a resource group or a subscription.

    Optional Parameters:
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az monitor log-analytics workspace list", locals())


def list_deleted_workspaces(resource_group=None):
    '''
    Get a list of deleted workspaces that can be recovered in a subscription or a resource group.

    Optional Parameters:
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az monitor log-analytics workspace list-deleted-workspaces", locals())


def recover(workspace_name, no_wait=None, resource_group=None):
    '''
    Recover a workspace in a soft-delete state within 14 days.

    Required Parameters:
    - workspace_name -- Name of the Log Analytics Workspace.

    Optional Parameters:
    - no_wait -- Do not wait for the long-running operation to finish.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az monitor log-analytics workspace recover", locals())


def get_schema(resource_group, workspace_name):
    '''
    Get the schema for a given workspace.

    Required Parameters:
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - workspace_name -- Name of the Log Analytics Workspace.
    '''
    return _call_az("az monitor log-analytics workspace get-schema", locals())


def list_usages(resource_group, workspace_name):
    '''
    Get a list of usage metrics for a workspace.

    Required Parameters:
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - workspace_name -- Name of the Log Analytics Workspace.
    '''
    return _call_az("az monitor log-analytics workspace list-usages", locals())


def list_management_groups(resource_group, workspace_name):
    '''
    Get a list of management groups connected to a workspace.

    Required Parameters:
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - workspace_name -- Name of the Log Analytics Workspace.
    '''
    return _call_az("az monitor log-analytics workspace list-management-groups", locals())


def get_shared_keys(resource_group, workspace_name):
    '''
    Get the shared keys for a workspace.

    Required Parameters:
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - workspace_name -- Name of the Log Analytics Workspace.
    '''
    return _call_az("az monitor log-analytics workspace get-shared-keys", locals())




'''
manage shared image gallery
'''
from .. pyaz_utils import _call_az
from . import gallery_application, image_definition, image_version, share


def create(gallery_name, resource_group, description=None, location=None, permissions=None, soft_delete=None, tags=None):
    '''
    Create a shared image gallery.

    Required Parameters:
    - gallery_name -- gallery name
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - description -- the description of the gallery
    - location -- Location. Values from: `az account list-locations`. You can configure the default location using `az configure --defaults location=<location>`.
    - permissions -- This property allows you to specify the permission of sharing gallery.
    - soft_delete -- Enable soft-deletion for resources in this gallery, allowing them to be recovered within retention time.
    - tags -- space-separated tags: key[=value] [key[=value] ...]. Use '' to clear existing tags.
    '''
    return _call_az("az sig create", locals())


def show(gallery_name, resource_group, select=None):
    '''
    

    Required Parameters:
    - gallery_name -- gallery name
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - select -- The select expression to apply on the operation.
    '''
    return _call_az("az sig show", locals())


def list(resource_group=None):
    '''
    list share image galleries.

    Optional Parameters:
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az sig list", locals())


def delete(gallery_name, resource_group):
    '''
    

    Required Parameters:
    - gallery_name -- gallery name
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az sig delete", locals())


def update(gallery_name, resource_group, add=None, force_string=None, permissions=None, remove=None, select=None, set=None, soft_delete=None):
    '''
    update a share image gallery.

    Required Parameters:
    - gallery_name -- gallery name
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - add -- Add an object to a list of objects by specifying a path and key value pairs.  Example: --add property.listProperty <key=value, string or JSON string>
    - force_string -- When using 'set' or 'add', preserve string literals instead of attempting to convert to JSON.
    - permissions -- This property allows you to specify the permission of sharing gallery.
    - remove -- Remove a property or an element from a list.  Example: --remove property.list <indexToRemove> OR --remove propertyToRemove
    - select -- The select expression to apply on the operation.
    - set -- Update an object by specifying a property path and value to set.  Example: --set property1.property2=<value>
    - soft_delete -- Enable soft-deletion for resources in this gallery, allowing them to be recovered within retention time.
    '''
    return _call_az("az sig update", locals())


def list_shared(location, shared_to=None):
    '''
    List all galleries shared directly to your subscription or tenant (preview).

    Required Parameters:
    - location -- Location. Values from: `az account list-locations`. You can configure the default location using `az configure --defaults location=<location>`.

    Optional Parameters:
    - shared_to -- The query parameter to decide what shared galleries to fetch when doing listing operations. If not specified, list by subscription id.
    '''
    return _call_az("az sig list-shared", locals())


def show_shared(gallery_unique_name, location):
    '''
    Get a gallery that has been shared directly to your subscription or tenant (preview).

    Required Parameters:
    - gallery_unique_name -- The unique name of the Shared Gallery.
    - location -- Location. Values from: `az account list-locations`. You can configure the default location using `az configure --defaults location=<location>`.
    '''
    return _call_az("az sig show-shared", locals())




'''
Manage VMs in an Azure DevTest Lab.
'''
from ... pyaz_utils import _call_az

def show(lab_name, name, resource_group, expand=None):
    '''
    

    Required Parameters:
    - lab_name -- The name of the lab.
    - name -- The name of the virtual machine.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - expand -- Specify the $expand query. Example: 'properties($expand=artifacts,computeVm,networkInterface,applicableSchedule)'
    '''
    return _call_az("az lab vm show", locals())


def delete(lab_name, name, resource_group):
    '''
    

    Required Parameters:
    - lab_name -- The name of the lab.
    - name -- The name of the virtual machine.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az lab vm delete", locals())


def start(lab_name, name, resource_group):
    '''
    

    Required Parameters:
    - lab_name -- The name of the lab.
    - name -- The name of the virtual machine.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az lab vm start", locals())


def stop(lab_name, name, resource_group):
    '''
    

    Required Parameters:
    - lab_name -- The name of the lab.
    - name -- The name of the virtual machine.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az lab vm stop", locals())


def apply_artifacts(lab_name, name, resource_group, artifacts=None):
    '''
    Apply artifacts to a virtual machine in Azure DevTest Lab.

    Required Parameters:
    - lab_name -- The name of the lab.
    - name -- The name of the virtual machine.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - artifacts -- The list of artifacts to apply.
    '''
    return _call_az("az lab vm apply-artifacts", locals())


def list(lab_name, resource_group, all=None, claimable=None, environment=None, expand=None, filters=None, object_id=None, order_by=None, top=None):
    '''
    List the VMs in an Azure DevTest Lab.

    Required Parameters:
    - lab_name -- None
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - all -- None
    - claimable -- None
    - environment -- None
    - expand -- None
    - filters -- None
    - object_id -- None
    - order_by -- None
    - top -- None
    '''
    return _call_az("az lab vm list", locals())


def claim(lab_name=None, name=None, resource_group=None):
    '''
    Claim a virtual machine from the Lab.

    Optional Parameters:
    - lab_name -- None
    - name -- None
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az lab vm claim", locals())


def create(lab_name, name, resource_group, admin_password=None, admin_username=None, allow_claim=None, artifacts=None, authentication_type=None, disk_type=None, expiration_date=None, formula=None, generate_ssh_keys=None, image=None, image_type=None, ip_configuration=None, notes=None, saved_secret=None, size=None, ssh_key=None, subnet=None, tags=None, vnet_name=None):
    '''
    Create a VM in a lab.

    Required Parameters:
    - lab_name -- None
    - name -- None
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - admin_password -- None
    - admin_username -- None
    - allow_claim -- None
    - artifacts -- None
    - authentication_type -- None
    - disk_type -- None
    - expiration_date -- None
    - formula -- None
    - generate_ssh_keys -- None
    - image -- None
    - image_type -- None
    - ip_configuration -- None
    - notes -- None
    - saved_secret -- None
    - size -- None
    - ssh_key -- None
    - subnet -- None
    - tags -- None
    - vnet_name -- None
    '''
    return _call_az("az lab vm create", locals())




'''
Manage Azure Resource Manager template deployment at subscription scope.
'''
from .. pyaz_utils import _call_az
from . import group, mg, operation, sub, tenant


def list(filter=None):
    '''
    List deployments at subscription scope.

    Optional Parameters:
    - filter -- Filter expression using OData notation. You can use --filter "provisioningState eq '{state}'" to filter provisioningState. To get more information, please visit https://docs.microsoft.com/rest/api/resources/deployments/listatsubscriptionscope#uri-parameters
    '''
    return _call_az("az deployment list", locals())


def show(name):
    '''
    Show a deployment at subscription scope.

    Required Parameters:
    - name -- The deployment name.
    '''
    return _call_az("az deployment show", locals())


def delete(name, no_wait=None):
    '''
    Delete a deployment at subscription scope.

    Required Parameters:
    - name -- The deployment name.

    Optional Parameters:
    - no_wait -- Do not wait for the long-running operation to finish.
    '''
    return _call_az("az deployment delete", locals())


def validate(location, handle_extended_json_format=None, name=None, no_prompt=None, parameters=None, query_string=None, template_file=None, template_spec=None, template_uri=None):
    '''
    Validate whether a template is valid at subscription scope.

    Required Parameters:
    - location -- Location. Values from: `az account list-locations`. You can configure the default location using `az configure --defaults location=<location>`.

    Optional Parameters:
    - handle_extended_json_format -- Support to handle extended template content including multiline and comments in deployment
    - name -- The deployment name. Default to template file base name
    - no_prompt -- The option to disable the prompt of missing parameters for ARM template. When the value is true, the prompt requiring users to provide missing parameter will be ignored. The default value is false.
    - parameters -- the deployment parameters
    - query_string -- The query string (a SAS token) to be used with the template-uri in the case of linked templates.
    - template_file -- a path to a template file or Bicep file in the file system
    - template_spec -- The template spec resource id.
    - template_uri -- a uri to a remote template file
    '''
    return _call_az("az deployment validate", locals())


def create(location, confirm_with_what_if=None, handle_extended_json_format=None, name=None, no_prompt=None, no_wait=None, parameters=None, proceed_if_no_change=None, query_string=None, template_file=None, template_spec=None, template_uri=None, what_if=None, what_if_exclude_change_types=None, what_if_result_format=None):
    '''
    Start a deployment at subscription scope.

    Required Parameters:
    - location -- Location. Values from: `az account list-locations`. You can configure the default location using `az configure --defaults location=<location>`.

    Optional Parameters:
    - confirm_with_what_if -- Instruct the command to run deployment What-If before executing the deployment. It then prompts you to acknowledge resource changes before it continues.
    - handle_extended_json_format -- Support to handle extended template content including multiline and comments in deployment
    - name -- The deployment name. Default to template file base name
    - no_prompt -- The option to disable the prompt of missing parameters for ARM template. When the value is true, the prompt requiring users to provide missing parameter will be ignored. The default value is false.
    - no_wait -- Do not wait for the long-running operation to finish.
    - parameters -- the deployment parameters
    - proceed_if_no_change -- Instruct the command to execute the deployment if the What-If result contains no resource changes. Applicable when --confirm-with-what-if is set.
    - query_string -- The query string (a SAS token) to be used with the template-uri in the case of linked templates.
    - template_file -- a path to a template file or Bicep file in the file system
    - template_spec -- The template spec resource id.
    - template_uri -- a uri to a remote template file
    - what_if -- Instruct the command to run deployment What-If.
    - what_if_exclude_change_types -- Space-separated list of resource change types to be excluded from What-If results. Applicable when --confirm-with-what-if is set.
    - what_if_result_format -- None
    '''
    return _call_az("az deployment create", locals())


def export(name):
    '''
    Export the template used for a deployment.

    Required Parameters:
    - name -- The deployment name.
    '''
    return _call_az("az deployment export", locals())


def wait(name, created=None, custom=None, deleted=None, exists=None, interval=None, timeout=None, updated=None):
    '''
    Place the CLI in a waiting state until a deployment condition is met.

    Required Parameters:
    - name -- The deployment name.

    Optional Parameters:
    - created -- wait until created with 'provisioningState' at 'Succeeded'
    - custom -- Wait until the condition satisfies a custom JMESPath query. E.g. provisioningState!='InProgress', instanceView.statuses[?code=='PowerState/running']
    - deleted -- wait until deleted
    - exists -- wait until the resource exists
    - interval -- polling interval in seconds
    - timeout -- maximum wait in seconds
    - updated -- wait until updated with provisioningState at 'Succeeded'
    '''
    return _call_az("az deployment wait", locals())


def cancel(name):
    '''
    Cancel a deployment at subscription scope.

    Required Parameters:
    - name -- The deployment name.
    '''
    return _call_az("az deployment cancel", locals())




'''
Manage file system access and permissions for Azure Data Lake Storage Gen2 account.
'''
from .... pyaz_utils import _call_az

def set(account_key=None, account_name=None, acl=None, auth_mode=None, connection_string=None, group=None, owner=None, permissions=None, sas_token=None):
    '''
    Set the access control properties of a path(directory or file) in Azure Data Lake Storage Gen2 account.

    Optional Parameters:
    - account_key -- Storage account key. Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_KEY
    - account_name -- Storage account name. Related environment variable: AZURE_STORAGE_ACCOUNT. Must be used in conjunction with either storage account key or a SAS token. If neither are present, the command will try to query the storage account key using the authenticated Azure account. If a large number of storage commands are executed the API quota may be hit
    - acl --  Sets POSIX access control rights on files and directories. The value is a comma-separated list of access control entries. Each access control entry (ACE) consists of a scope, a type, a user or group identifier, and permissions in the format "[scope:][type]:[id]:[permissions]". permissions and acl are mutually exclusive.
    - auth_mode -- The mode in which to run the command. "login" mode will directly use your login credentials for the authentication. The legacy "key" mode will attempt to query for an account key if no authentication parameters for the account are provided. Environment variable: AZURE_STORAGE_AUTH_MODE
    - connection_string -- Storage account connection string. Environment variable: AZURE_STORAGE_CONNECTION_STRING
    - group --  Optional. The owning group of the file or directory.
    - owner --  Optional. The owner of the file or directory.
    - permissions --  Optional and only valid if Hierarchical Namespace is enabled for the account. Sets POSIX access permissions for the file owner, the file owning group, and others. Each class may be granted read, write, or execute permission.  The sticky bit is also supported. Both symbolic (rwxrw-rw-) and 4-digit octal notation (e.g. 0766) are supported. permissions and acl are mutually exclusive.
    - sas_token -- A Shared Access Signature (SAS). Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_SAS_TOKEN
    '''
    return _call_az("az storage fs access set", locals())


def show(account_key=None, account_name=None, auth_mode=None, connection_string=None, sas_token=None):
    '''
    Show the access control properties of a path (directory or file) in Azure Data Lake Storage Gen2 account.

    Optional Parameters:
    - account_key -- Storage account key. Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_KEY
    - account_name -- Storage account name. Related environment variable: AZURE_STORAGE_ACCOUNT. Must be used in conjunction with either storage account key or a SAS token. If neither are present, the command will try to query the storage account key using the authenticated Azure account. If a large number of storage commands are executed the API quota may be hit
    - auth_mode -- The mode in which to run the command. "login" mode will directly use your login credentials for the authentication. The legacy "key" mode will attempt to query for an account key if no authentication parameters for the account are provided. Environment variable: AZURE_STORAGE_AUTH_MODE
    - connection_string -- Storage account connection string. Environment variable: AZURE_STORAGE_CONNECTION_STRING
    - sas_token -- A Shared Access Signature (SAS). Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_SAS_TOKEN
    '''
    return _call_az("az storage fs access show", locals())


def set_recursive(acl, account_key=None, account_name=None, auth_mode=None, batch_size=None, connection_string=None, continuation=None, continue_on_failure=None, max_batches=None, sas_token=None, timeout=None):
    '''
    Set the Access Control on a path and sub-paths in Azure Data Lake Storage Gen2 account.

    Required Parameters:
    - acl -- The value is a comma-separated list of access control entries. Each access control entry (ACE) consists of a scope, a type, a user or group identifier, and permissions in the format "[scope:][type]:[id]:[permissions]".  For more information, please refer to https://docs.microsoft.com/azure/storage/blobs/data-lake-storage-access-control.

    Optional Parameters:
    - account_key -- Storage account key. Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_KEY
    - account_name -- Storage account name. Related environment variable: AZURE_STORAGE_ACCOUNT. Must be used in conjunction with either storage account key or a SAS token. If neither are present, the command will try to query the storage account key using the authenticated Azure account. If a large number of storage commands are executed the API quota may be hit
    - auth_mode -- The mode in which to run the command. "login" mode will directly use your login credentials for the authentication. The legacy "key" mode will attempt to query for an account key if no authentication parameters for the account are provided. Environment variable: AZURE_STORAGE_AUTH_MODE
    - batch_size -- Optional. If data set size exceeds batch size then operation will be split into multiple requests so that progress can be tracked. Batch size should be between 1 and 2000. The default when unspecified is 2000.
    - connection_string -- Storage account connection string. Environment variable: AZURE_STORAGE_CONNECTION_STRING
    - continuation -- Optional continuation token that can be used to resume previously stopped operation.
    - continue_on_failure -- If set to False, the operation will terminate quickly on encountering user errors (4XX). If True, the operation will ignore user errors and proceed with the operation on other sub-entities of the directory. Continuation token will only be returned when --continue-on-failure is True in case of user errors. If not set the default value is False for this.
    - max_batches -- Optional. Define maximum number of batches that single change Access Control operation can execute. If maximum is reached before all sub-paths are processed, then continuation token can be used to resume operation. Empty value indicates that maximum number of batches in unbound and operation continues till end.
    - sas_token -- A Shared Access Signature (SAS). Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_SAS_TOKEN
    - timeout -- Request timeout in seconds. Applies to each call to the service.
    '''
    return _call_az("az storage fs access set-recursive", locals())


def update_recursive(acl, account_key=None, account_name=None, auth_mode=None, batch_size=None, connection_string=None, continuation=None, continue_on_failure=None, max_batches=None, sas_token=None, timeout=None):
    '''
    Modify the Access Control on a path and sub-paths in Azure Data Lake Storage Gen2 account.

    Required Parameters:
    - acl -- The value is a comma-separated list of access control entries. Each access control entry (ACE) consists of a scope, a type, a user or group identifier, and permissions in the format "[scope:][type]:[id]:[permissions]".  For more information, please refer to https://docs.microsoft.com/azure/storage/blobs/data-lake-storage-access-control.

    Optional Parameters:
    - account_key -- Storage account key. Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_KEY
    - account_name -- Storage account name. Related environment variable: AZURE_STORAGE_ACCOUNT. Must be used in conjunction with either storage account key or a SAS token. If neither are present, the command will try to query the storage account key using the authenticated Azure account. If a large number of storage commands are executed the API quota may be hit
    - auth_mode -- The mode in which to run the command. "login" mode will directly use your login credentials for the authentication. The legacy "key" mode will attempt to query for an account key if no authentication parameters for the account are provided. Environment variable: AZURE_STORAGE_AUTH_MODE
    - batch_size -- Optional. If data set size exceeds batch size then operation will be split into multiple requests so that progress can be tracked. Batch size should be between 1 and 2000. The default when unspecified is 2000.
    - connection_string -- Storage account connection string. Environment variable: AZURE_STORAGE_CONNECTION_STRING
    - continuation -- Optional continuation token that can be used to resume previously stopped operation.
    - continue_on_failure -- If set to False, the operation will terminate quickly on encountering user errors (4XX). If True, the operation will ignore user errors and proceed with the operation on other sub-entities of the directory. Continuation token will only be returned when --continue-on-failure is True in case of user errors. If not set the default value is False for this.
    - max_batches -- Optional. Define maximum number of batches that single change Access Control operation can execute. If maximum is reached before all sub-paths are processed, then continuation token can be used to resume operation. Empty value indicates that maximum number of batches in unbound and operation continues till end.
    - sas_token -- A Shared Access Signature (SAS). Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_SAS_TOKEN
    - timeout -- Request timeout in seconds. Applies to each call to the service.
    '''
    return _call_az("az storage fs access update-recursive", locals())


def remove_recursive(acl, account_key=None, account_name=None, auth_mode=None, batch_size=None, connection_string=None, continuation=None, continue_on_failure=None, max_batches=None, sas_token=None, timeout=None):
    '''
    Remove the Access Control on a path and sub-paths in Azure Data Lake Storage Gen2 account.

    Required Parameters:
    - acl -- The value is a comma-separated list of access control entries. Each access control entry (ACE) consists of a scope, a type, a user or group identifier, and permissions in the format "[scope:][type]:[id]:[permissions]".  For more information, please refer to https://docs.microsoft.com/azure/storage/blobs/data-lake-storage-access-control.

    Optional Parameters:
    - account_key -- Storage account key. Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_KEY
    - account_name -- Storage account name. Related environment variable: AZURE_STORAGE_ACCOUNT. Must be used in conjunction with either storage account key or a SAS token. If neither are present, the command will try to query the storage account key using the authenticated Azure account. If a large number of storage commands are executed the API quota may be hit
    - auth_mode -- The mode in which to run the command. "login" mode will directly use your login credentials for the authentication. The legacy "key" mode will attempt to query for an account key if no authentication parameters for the account are provided. Environment variable: AZURE_STORAGE_AUTH_MODE
    - batch_size -- Optional. If data set size exceeds batch size then operation will be split into multiple requests so that progress can be tracked. Batch size should be between 1 and 2000. The default when unspecified is 2000.
    - connection_string -- Storage account connection string. Environment variable: AZURE_STORAGE_CONNECTION_STRING
    - continuation -- Optional continuation token that can be used to resume previously stopped operation.
    - continue_on_failure -- If set to False, the operation will terminate quickly on encountering user errors (4XX). If True, the operation will ignore user errors and proceed with the operation on other sub-entities of the directory. Continuation token will only be returned when --continue-on-failure is True in case of user errors. If not set the default value is False for this.
    - max_batches -- Optional. Define maximum number of batches that single change Access Control operation can execute. If maximum is reached before all sub-paths are processed, then continuation token can be used to resume operation. Empty value indicates that maximum number of batches in unbound and operation continues till end.
    - sas_token -- A Shared Access Signature (SAS). Must be used in conjunction with storage account name. Environment variable: AZURE_STORAGE_SAS_TOKEN
    - timeout -- Request timeout in seconds. Applies to each call to the service.
    '''
    return _call_az("az storage fs access remove-recursive", locals())




from .... pyaz_utils import _call_az

def setup(admin_password, admin_user, database_name, repo, resource_group, server_name, sql_file, action_name=None, allow_push=None, branch=None):
    '''
    Create github action workflow file for MySQL server.

    Required Parameters:
    - admin_password -- The password of the administrator. Minimum 8 characters and maximum 128 characters. Password must contain characters from three of the following categories: English uppercase letters, English lowercase letters, numbers, and non-alphanumeric characters.
    - admin_user -- Administrator username for the server. Once set, it cannot be changed. 
    - database_name -- The name of the database to be created when provisioning the database server
    - repo -- The name of your github username and repository e.g., Azure/azure-cli 
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - server_name -- Name of the server. The name can contain only lowercase letters, numbers, and the hyphen (-) character. Minimum 3 characters and maximum 63 characters.
    - sql_file -- The path of the sql file. The sql file should be already in the repository

    Optional Parameters:
    - action_name -- The name of the github action
    - allow_push -- Push the action yml file to the remote repository. The changes will be pushed to origin repository, speicified branch or current branch if not specified.
    - branch -- The name of the branch you want upload github action file. The default will be your current branch.
    '''
    return _call_az("az mysql flexible-server deploy setup", locals())


def run(action_name, branch):
    '''
    Run an existing workflow in your github repository

    Required Parameters:
    - action_name -- The name of the github action
    - branch -- The name of the branch you want upload github action file. The default will be your current branch.
    '''
    return _call_az("az mysql flexible-server deploy run", locals())




'''
Manage permissions for a provider.
'''
from ... pyaz_utils import _call_az

def list(namespace):
    '''
    List permissions from a provider.

    Required Parameters:
    - namespace -- the resource namespace, aka 'provider'
    '''
    return _call_az("az provider permission list", locals())




'''
Manage replication between databases.
'''
from .... pyaz_utils import _call_az

def create(name, partner_server, resource_group, server, auto_pause_delay=None, backup_storage_redundancy=None, capacity=None, compute_model=None, elastic_pool=None, family=None, license_type=None, min_capacity=None, no_wait=None, partner_database=None, partner_resource_group=None, read_replicas=None, read_scale=None, secondary_type=None, service_objective=None, tags=None, zone_redundant=None):
    '''
    Create a database as a readable secondary replica of an existing database.

    Required Parameters:
    - name -- Name of the Azure SQL Database.
    - partner_server -- Name of the server to create the new replica in.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - server -- Name of the Azure SQL server. You can configure the default using `az configure --defaults sql-server=<name>`

    Optional Parameters:
    - auto_pause_delay -- Time in minutes after which database is automatically paused. A value of -1 means that automatic pause is disabled.
    - backup_storage_redundancy -- Backup storage redundancy used to store backups. Allowed values include: Local, Zone, Geo.
    - capacity -- The capacity component of the sku in integer number of DTUs or vcores.
    - compute_model -- The compute model of the database.
    - elastic_pool -- The name or resource id of the elastic pool to create the database in.
    - family -- The compute generation component of the sku (for vcore skus only). Allowed values include: Gen4, Gen5.
    - license_type -- The license type to apply for this database. ``LicenseIncluded`` if you need a license, or ``BasePrice`` if you have a license and are eligible for the Azure Hybrid Benefit. Possible values include: "LicenseIncluded", "BasePrice".
    - min_capacity -- Minimal capacity that database will always have allocated, if not paused
    - no_wait -- Do not wait for the long-running operation to finish.
    - partner_database -- Name of the new replica. If unspecified, defaults to the source database name.
    - partner_resource_group -- Name of the resource group to create the new replica in. If unspecified, defaults to the origin resource group.
    - read_replicas -- The number of high availability replicas to provision for the database. Only settable for Hyperscale edition.
    - read_scale -- If enabled, connections that have application intent set to readonly in their connection string may be routed to a readonly secondary replica. This property is only settable for Premium and Business Critical databases.
    - secondary_type -- Type of secondary to create. Allowed values include: Geo, Named.
    - service_objective -- The service objective for the new database. For example: Basic, S0, P1, GP_Gen4_1, GP_Gen5_S_8, BC_Gen5_2, HS_Gen5_32.
    - tags -- space-separated tags: key[=value] [key[=value] ...]. Use '' to clear existing tags.
    - zone_redundant -- Specifies whether to enable zone redundancy
    '''
    return _call_az("az sql db replica create", locals())


def list_links(name, resource_group, server):
    '''
    List the replicas of a database and their replication status.

    Required Parameters:
    - name -- Name of the Azure SQL Database.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - server -- Name of the Azure SQL server. You can configure the default using `az configure --defaults sql-server=<name>`
    '''
    return _call_az("az sql db replica list-links", locals())


def delete_link(name, partner_server, resource_group, server, partner_resource_group=None, yes=None):
    '''
    Permanently stop data replication between two database replicas.

    Required Parameters:
    - name -- Name of the Azure SQL Database.
    - partner_server -- Name of the server that the other replica is in.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - server -- Name of the Azure SQL server. You can configure the default using `az configure --defaults sql-server=<name>`

    Optional Parameters:
    - partner_resource_group -- Name of the resource group that the other replica is in. If unspecified, defaults to the first database's resource group.
    - yes -- Do not prompt for confirmation.
    '''
    return _call_az("az sql db replica delete-link", locals())


def set_primary(name, resource_group, server, allow_data_loss=None):
    '''
    Set the primary replica database by failing over from the current primary replica database.

    Required Parameters:
    - name -- Name of the database to fail over.
    - resource_group -- Name of the resource group containing the secondary replica that will become the new primary.
    - server -- Name of the server containing the secondary replica that will become the new primary. You can configure the default using `az configure --defaults sql-server=<name>`

    Optional Parameters:
    - allow_data_loss -- If specified, the failover operation will allow data loss.
    '''
    return _call_az("az sql db replica set-primary", locals())




'''
Commands to work with cluster nodes.
'''
from .... pyaz_utils import _call_az

def list(cluster, resource_group, workspace):
    '''
    List remote login information for cluster's nodes.

    Required Parameters:
    - cluster -- Name of cluster.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - workspace -- Name of workspace.
    '''
    return _call_az("az batchai cluster node list", locals())


def exec(cluster, resource_group, workspace, address=None, exec=None, node_id=None, password=None, ssh_private_key=None):
    '''
    Executes a command line on a cluster's node with optional ports forwarding.

    Required Parameters:
    - cluster -- Name of cluster.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - workspace -- Name of workspace.

    Optional Parameters:
    - address -- Specifies that connections to the given TCP port or Unix socket on the local (client) host are to be forwarded to the given host and port, or Unix socket, on the remote side. e.g. -L 8080:localhost:8080
    - exec -- Optional command line to be executed on the node. If not provided, the command will perform ports forwarding only.
    - node_id -- ID of the node to forward the ports to. If not provided, the command will be executed on the first available node.
    - password -- Optional password to establish SSH connection.
    - ssh_private_key -- Optional SSH private key path to establish SSH connection. If omitted, the default SSH private key will be used.
    '''
    return _call_az("az batchai cluster node exec", locals())




'''
Manage MongoDB resources of Azure Cosmos DB account.
'''
from ... pyaz_utils import _call_az
from . import collection, database, restorable_collection, restorable_database, restorable_resource


def retrieve_latest_backup_time(account_name, collection_name, database_name, location, resource_group):
    '''
    Retrieves latest restorable timestamp for the given mongodb collection in given region.

    Required Parameters:
    - account_name -- Name of the CosmosDB database account
    - collection_name -- Name of the CosmosDB MongoDB collection name
    - database_name -- Name of the CosmosDB MongoDB database name
    - location -- Location of the account
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az cosmosdb mongodb retrieve-latest-backup-time", locals())




'''
Manage resource policy assignments.
'''
from ... pyaz_utils import _call_az
from . import identity, non_compliance_message


def create(assign_identity=None, description=None, display_name=None, enforcement_mode=None, identity_scope=None, location=None, mi_system_assigned=None, mi_user_assigned=None, name=None, not_scopes=None, params=None, policy=None, policy_set_definition=None, resource_group=None, role=None, scope=None, sku=None):
    '''
    Create a resource policy assignment.

    Optional Parameters:
    - assign_identity -- Assigns a system assigned identity to the policy assignment. This argument will be deprecated, please use --mi-system-assigned instead
    - description -- Description of the policy assignment.
    - display_name -- Display name of the policy assignment.
    - enforcement_mode -- Enforcement mode of the policy assignment, e.g. Default, DoNotEnforce. Please visit https://aka.ms/azure-policyAssignment-enforcement-mode for more information.
    - identity_scope -- Scope that the system assigned identity can access
    - location -- The location of the policy assignment. Only required when utilizing managed identity.
    - mi_system_assigned -- Provide this flag to use system assigned identity for policy assignment. Check out help for more examples
    - mi_user_assigned -- UserAssigned Identity Id to be used for policy assignment. Check out help for more examples
    - name -- Name of the new policy assignment.
    - not_scopes -- Space-separated scopes where the policy assignment does not apply.
    - params -- JSON formatted string or a path to a file or uri with parameter values of the policy rule.
    - policy -- Name or id of the policy definition.
    - policy_set_definition -- Name or id of the policy set definition.
    - resource_group -- the resource group where the policy will be applied
    - role -- Role name or id that will be assigned to the managed identity
    - scope -- Scope to which this policy assignment applies.
    - sku -- policy sku.
    '''
    return _call_az("az policy assignment create", locals())


def delete(name, resource_group=None, scope=None):
    '''
    Delete a resource policy assignment.

    Required Parameters:
    - name -- Name of the policy assignment.

    Optional Parameters:
    - resource_group -- the resource group where the policy will be applied
    - scope -- Scope to which this policy assignment applies.
    '''
    return _call_az("az policy assignment delete", locals())


def list(disable_scope_strict_match=None, resource_group=None, scope=None):
    '''
    List resource policy assignments.

    Optional Parameters:
    - disable_scope_strict_match -- Include policy assignments either inherited from parent scope or at child scope.
    - resource_group -- the resource group where the policy will be applied
    - scope -- Scope to which this policy assignment applies.
    '''
    return _call_az("az policy assignment list", locals())


def show(name, resource_group=None, scope=None):
    '''
    Show a resource policy assignment.

    Required Parameters:
    - name -- Name of the policy assignment.

    Optional Parameters:
    - resource_group -- the resource group where the policy will be applied
    - scope -- Scope to which this policy assignment applies.
    '''
    return _call_az("az policy assignment show", locals())


def update(description=None, display_name=None, enforcement_mode=None, name=None, not_scopes=None, params=None, resource_group=None, scope=None, sku=None):
    '''
    Update a resource policy assignment.

    Optional Parameters:
    - description -- Description of the policy assignment.
    - display_name -- Display name of the policy assignment.
    - enforcement_mode -- Enforcement mode of the policy assignment, e.g. Default, DoNotEnforce. Please visit https://aka.ms/azure-policyAssignment-enforcement-mode for more information.
    - name -- Name of the policy assignment.
    - not_scopes -- Space-separated scopes where the policy assignment does not apply.
    - params -- JSON formatted string or a path to a file or uri with parameter values of the policy rule.
    - resource_group -- the resource group where the policy will be applied
    - scope -- Scope to which this policy assignment applies.
    - sku -- policy sku.
    '''
    return _call_az("az policy assignment update", locals())




from ... pyaz_utils import _call_az

def list():
    '''
    List supported of secure score controls details and state for scope.
    '''
    return _call_az("az security secure-score-controls list", locals())


def list_by_score(name):
    '''
    List supported of secure score controls details and state for selected score.

    Required Parameters:
    - name -- name of the resource to be fetched
    '''
    return _call_az("az security secure-score-controls list_by_score", locals())




'''
Manage an application's OAuth2 permissions.
'''
from .... pyaz_utils import _call_az

def grant(api, id, consent_type=None, expires=None, principal_id=None, scope=None):
    '''
    Grant the app an API Delegated permissions

    Required Parameters:
    - api -- Specify `RequiredResourceAccess.resourceAppId` - The unique identifier for the resource that the application requires access to. This should be equal to the appId declared on the target resource application.
    - id -- identifier uri, application id, or object id

    Optional Parameters:
    - consent_type -- Indicates if consent was provided by the administrator (on behalf of the organization) or by an individual.
    - expires -- Expiry date for the permissions in years. e.g. 1, 2 or "never"
    - principal_id -- If --consent-type is "Principal", this argument specifies the object of the user that granted consent and applies only for that user.
    - scope -- Specifies the value of the scope claim that the resource application should expect in the OAuth 2.0 access token, e.g. User.Read
    '''
    return _call_az("az ad app permission grant", locals())


def list(id):
    '''
    List API permissions the application has requested

    Required Parameters:
    - id -- identifier uri, application id, or object id of the associated application
    '''
    return _call_az("az ad app permission list", locals())


def add(api, api_permissions, id):
    '''
    Add an API permission

    Required Parameters:
    - api -- Specify `RequiredResourceAccess.resourceAppId` - The unique identifier for the resource that the application requires access to. This should be equal to the appId declared on the target resource application.
    - api_permissions -- Specify `ResourceAccess.id` - The unique identifier for one of the OAuth2Permission or AppRole instances that the resource application exposes. Space-separated list of `<resource-access-id>=<type>`.
    - id -- identifier uri, application id, or object id
    '''
    return _call_az("az ad app permission add", locals())


def delete(api, id, api_permissions=None):
    '''
    Remove an API permission

    Required Parameters:
    - api -- Specify `RequiredResourceAccess.resourceAppId` - The unique identifier for the resource that the application requires access to. This should be equal to the appId declared on the target resource application.
    - id -- identifier uri, application id, or object id

    Optional Parameters:
    - api_permissions -- Specify `ResourceAccess.id` - The unique identifier for one of the OAuth2Permission or AppRole instances that the resource application exposes. Space-separated list of `<resource-access-id>`.
    '''
    return _call_az("az ad app permission delete", locals())


def list_grants(filter=None, id=None, show_resource_name=None):
    '''
    List Oauth2 permission grants

    Optional Parameters:
    - filter -- OData filter, e.g. --filter "displayname eq 'test' and servicePrincipalType eq 'Application'"
    - id -- identifier uri, application id, or object id
    - show_resource_name -- show resource's display name
    '''
    return _call_az("az ad app permission list-grants", locals())


def admin_consent(id):
    '''
    Grant Application & Delegated permissions through admin-consent.

    Required Parameters:
    - id -- identifier uri, application id, or object id
    '''
    return _call_az("az ad app permission admin-consent", locals())




'''
Manage web app deployment logs.
'''
from .... pyaz_utils import _call_az

def show(name, resource_group, deployment_id=None, slot=None):
    '''
    Show deployment logs of the latest deployment, or a specific deployment if deployment-id is specified.

    Required Parameters:
    - name -- name of the web app. If left unspecified, a name will be randomly generated. You can configure the default using `az configure --defaults web=<name>`
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - deployment_id -- Deployment ID. If none specified, returns the deployment logs of the latest deployment.
    - slot -- the name of the slot. Default to the productions slot if not specified
    '''
    return _call_az("az webapp log deployment show", locals())


def list(name, resource_group, slot=None):
    '''
    List deployments associated with web app

    Required Parameters:
    - name -- name of the web app. If left unspecified, a name will be randomly generated. You can configure the default using `az configure --defaults web=<name>`
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - slot -- the name of the slot. Default to the productions slot if not specified
    '''
    return _call_az("az webapp log deployment list", locals())




'''
Manage CDN WAF policies.
'''
from .... pyaz_utils import _call_az
from . import custom_rule, managed_rule_set, rate_limit_rule


def show(name, resource_group):
    '''
    Show details of a CDN WAF policy.

    Required Parameters:
    - name -- The name of the CDN WAF policy.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az cdn waf policy show", locals())


def list(resource_group):
    '''
    List CDN WAF policies.

    Required Parameters:
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az cdn waf policy list", locals())


def set(name, resource_group, block_response_body=None, block_response_status_code=None, disabled=None, mode=None, redirect_url=None, sku=None, tags=None):
    '''
    Create a new CDN WAF policy.

    Required Parameters:
    - name -- The name of the CDN WAF policy.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - block_response_body -- None
    - block_response_status_code -- None
    - disabled -- None
    - mode -- None
    - redirect_url -- None
    - sku -- None
    - tags -- space-separated tags: key[=value] [key[=value] ...]. Use '' to clear existing tags.
    '''
    return _call_az("az cdn waf policy set", locals())


def delete(name, resource_group, yes=None):
    '''
    Delete a CDN WAF policy.

    Required Parameters:
    - name -- The name of the CDN WAF policy.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - yes -- Do not prompt for confirmation.
    '''
    return _call_az("az cdn waf policy delete", locals())




from ... pyaz_utils import _call_az
from . import aad, ipsec_policy, nat_rule, packet_capture, revoked_cert, root_cert, vpn_client


def create(name, public_ip_addresses, resource_group, vnet, aad_audience=None, aad_issuer=None, aad_tenant=None, address_prefixes=None, asn=None, bgp_peering_address=None, client_protocol=None, custom_routes=None, edge_zone=None, gateway_default_site=None, gateway_type=None, location=None, nat_rule=None, no_wait=None, peer_weight=None, radius_secret=None, radius_server=None, root_cert_data=None, root_cert_name=None, sku=None, tags=None, vpn_auth_type=None, vpn_gateway_generation=None, vpn_type=None):
    '''
    Create a virtual network gateway.

    Required Parameters:
    - name -- Name of the VNet gateway.
    - public_ip_addresses -- Specify a single public IP (name or ID) for an active-standby gateway. Specify two space-separated public IPs for an active-active gateway.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - vnet -- Name or ID of an existing virtual network which has a subnet named 'GatewaySubnet'.

    Optional Parameters:
    - aad_audience -- The AADAudience ID of the VirtualNetworkGateway.
    - aad_issuer -- The AAD Issuer URI of the VirtualNetworkGateway.
    - aad_tenant -- The AAD Tenant URI of the VirtualNetworkGateway.
    - address_prefixes -- Space-separated list of CIDR prefixes representing the address space for the P2S Vpnclient.
    - asn -- Autonomous System Number to use for the BGP settings.
    - bgp_peering_address -- IP address to use for BGP peering.
    - client_protocol -- Protocols to use for connecting
    - custom_routes -- Space-separated list of CIDR prefixes representing the custom routes address space specified by the customer for VpnClient.
    - edge_zone -- The name of edge zone.
    - gateway_default_site -- Name or ID of a local network gateway representing a local network site with default routes.
    - gateway_type -- The gateway type.
    - location -- Location. Values from: `az account list-locations`. You can configure the default location using `az configure --defaults location=<location>`.
    - nat_rule -- None
    - no_wait -- Do not wait for the long-running operation to finish.
    - peer_weight -- Weight (0-100) added to routes learned through BGP peering.
    - radius_secret -- Radius secret to use for authentication.
    - radius_server -- Radius server address to connect to.
    - root_cert_data -- Base64 contents of the root certificate file or file path.
    - root_cert_name -- Root certificate name
    - sku -- VNet gateway SKU.
    - tags -- space-separated tags: key[=value] [key[=value] ...]. Use '' to clear existing tags.
    - vpn_auth_type -- VPN authentication types enabled for the virtual network gateway.
    - vpn_gateway_generation -- The generation for the virtual network gateway. vpn_gateway_generation should not be provided if gateway_type is not Vpn.
    - vpn_type -- VPN routing type.
    '''
    return _call_az("az network vnet-gateway create", locals())


def update(name, resource_group, aad_audience=None, aad_issuer=None, aad_tenant=None, add=None, address_prefixes=None, asn=None, bgp_peering_address=None, client_protocol=None, custom_routes=None, enable_bgp=None, force_string=None, gateway_default_site=None, gateway_type=None, no_wait=None, peer_weight=None, public_ip_addresses=None, radius_secret=None, radius_server=None, remove=None, root_cert_data=None, root_cert_name=None, set=None, sku=None, tags=None, vnet=None, vpn_auth_type=None, vpn_type=None):
    '''
    Update a virtual network gateway.

    Required Parameters:
    - name -- Name of the VNet gateway.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - aad_audience -- The AADAudience ID of the VirtualNetworkGateway.
    - aad_issuer -- The AAD Issuer URI of the VirtualNetworkGateway.
    - aad_tenant -- The AAD Tenant URI of the VirtualNetworkGateway.
    - add -- Add an object to a list of objects by specifying a path and key value pairs.  Example: --add property.listProperty <key=value, string or JSON string>
    - address_prefixes -- Space-separated list of CIDR prefixes representing the address space for the P2S Vpnclient.
    - asn -- Autonomous System Number to use for the BGP settings.
    - bgp_peering_address -- IP address to use for BGP peering.
    - client_protocol -- Protocols to use for connecting
    - custom_routes -- Space-separated list of CIDR prefixes representing the custom routes address space specified by the customer for VpnClient.
    - enable_bgp -- Enable BGP (Border Gateway Protocol)
    - force_string -- When using 'set' or 'add', preserve string literals instead of attempting to convert to JSON.
    - gateway_default_site -- Name or ID of a local network gateway representing a local network site with default routes.
    - gateway_type -- The gateway type.
    - no_wait -- Do not wait for the long-running operation to finish.
    - peer_weight -- Weight (0-100) added to routes learned through BGP peering.
    - public_ip_addresses -- Specify a single public IP (name or ID) for an active-standby gateway. Specify two space-separated public IPs for an active-active gateway.
    - radius_secret -- Radius secret to use for authentication.
    - radius_server -- Radius server address to connect to.
    - remove -- Remove a property or an element from a list.  Example: --remove property.list <indexToRemove> OR --remove propertyToRemove
    - root_cert_data -- Base64 contents of the root certificate file or file path.
    - root_cert_name -- Root certificate name
    - set -- Update an object by specifying a property path and value to set.  Example: --set property1.property2=<value>
    - sku -- VNet gateway SKU.
    - tags -- space-separated tags: key[=value] [key[=value] ...]. Use '' to clear existing tags.
    - vnet -- Name or ID of a virtual network that contains a subnet named 'GatewaySubnet'.
    - vpn_auth_type -- VPN authentication types enabled for the virtual network gateway.
    - vpn_type -- VPN routing type.
    '''
    return _call_az("az network vnet-gateway update", locals())


def wait(name, resource_group, created=None, custom=None, deleted=None, exists=None, interval=None, timeout=None, updated=None):
    '''
    Place the CLI in a waiting state until a condition of the virtual network gateway is met.

    Required Parameters:
    - name -- Name of the VNet gateway.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - created -- wait until created with 'provisioningState' at 'Succeeded'
    - custom -- Wait until the condition satisfies a custom JMESPath query. E.g. provisioningState!='InProgress', instanceView.statuses[?code=='PowerState/running']
    - deleted -- wait until deleted
    - exists -- wait until the resource exists
    - interval -- polling interval in seconds
    - timeout -- maximum wait in seconds
    - updated -- wait until updated with provisioningState at 'Succeeded'
    '''
    return _call_az("az network vnet-gateway wait", locals())


def delete(name, resource_group, no_wait=None):
    '''
    Delete a virtual network gateway.

    Required Parameters:
    - name -- Name of the VNet gateway.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - no_wait -- Do not wait for the long-running operation to finish.
    '''
    return _call_az("az network vnet-gateway delete", locals())


def show(name, resource_group):
    '''
    Get the details of a virtual network gateway.

    Required Parameters:
    - name -- Name of the VNet gateway.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az network vnet-gateway show", locals())


def list(resource_group):
    '''
    List virtual network gateways.

    Required Parameters:
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az network vnet-gateway list", locals())


def reset(name, resource_group, gateway_vip=None):
    '''
    Reset a virtual network gateway.

    Required Parameters:
    - name -- Name of the VNet gateway.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - gateway_vip -- Virtual network gateway vip address supplied to the begin reset of the active-active feature enabled gateway.
    '''
    return _call_az("az network vnet-gateway reset", locals())


def list_bgp_peer_status(name, resource_group, peer=None):
    '''
    Retrieve the status of BGP peers.

    Required Parameters:
    - name -- Name of the VNet gateway.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`

    Optional Parameters:
    - peer -- The IP address of the peer to retrieve the status of.
    '''
    return _call_az("az network vnet-gateway list-bgp-peer-status", locals())


def list_advertised_routes(name, peer, resource_group):
    '''
    List the routes of a virtual network gateway advertised to the specified peer.

    Required Parameters:
    - name -- Name of the VNet gateway.
    - peer -- The IP address of the peer.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az network vnet-gateway list-advertised-routes", locals())


def list_learned_routes(name, resource_group):
    '''
    This operation retrieves a list of routes the virtual network gateway has learned, including routes learned from BGP peers.

    Required Parameters:
    - name -- Name of the VNet gateway.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az network vnet-gateway list-learned-routes", locals())


def show_supported_devices(name, resource_group):
    '''
    Get a xml format representation for supported vpn devices.

    Required Parameters:
    - name -- Name of the VNet gateway.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    '''
    return _call_az("az network vnet-gateway show-supported-devices", locals())


def disconnect_vpn_connections(name, resource_group, vpn_connections, no_wait=None):
    '''
    Disconnect vpn connections of virtual network gateway.

    Required Parameters:
    - name -- Name of the VNet gateway.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - vpn_connections -- List of Name or ID of VPN connections.

    Optional Parameters:
    - no_wait -- Do not wait for the long-running operation to finish.
    '''
    return _call_az("az network vnet-gateway disconnect-vpn-connections", locals())




from ... pyaz_utils import _call_az

def show(pe_name, workspace_name):
    '''
    Get a synapse managed private endpoints.

    Required Parameters:
    - pe_name -- The managed private endpoint name.
    - workspace_name -- The workspace name.
    '''
    return _call_az("az synapse managed-private-endpoints show", locals())


def create(group_Id, pe_name, resource_id, workspace_name):
    '''
    Create a synapse managed private endpoints.

    Required Parameters:
    - group_Id -- The groupId to which the managed private endpoint is created
    - pe_name -- The managed private endpoint name.
    - resource_id -- The ARM resource ID of the resource to which the managed private endpoint is created. Ex - /subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/{resourceType}/{resourceName}
    - workspace_name -- The workspace name.
    '''
    return _call_az("az synapse managed-private-endpoints create", locals())


def list(workspace_name):
    '''
    List synapse managed private endpoints in a workspace.

    Required Parameters:
    - workspace_name -- The workspace name.
    '''
    return _call_az("az synapse managed-private-endpoints list", locals())


def delete(pe_name, workspace_name, yes=None):
    '''
    delete synapse managed private endpoints in a workspace.

    Required Parameters:
    - pe_name -- The managed private endpoint name.
    - workspace_name -- The workspace name.

    Optional Parameters:
    - yes -- Do not prompt for confirmation.
    '''
    return _call_az("az synapse managed-private-endpoints delete", locals())




'''
Manage recovery configuration of an Azure workload backed up item.
'''
from ... pyaz_utils import _call_az

def show(container_name, item_name, resource_group, restore_mode, vault_name, backup_management_type=None, filepath=None, from_full_rp_name=None, log_point_in_time=None, rp_name=None, target_container_name=None, target_item_name=None, target_resource_group=None, target_server_name=None, target_server_type=None, target_vault_name=None, workload_type=None):
    '''
    Construct the recovery configuration of an Azure workload backed up item.

    Required Parameters:
    - container_name -- Name of the backup container. Accepts 'Name' or 'FriendlyName' from the output of az backup container list command. If 'FriendlyName' is passed then BackupManagementType is required.
    - item_name -- Name of the backed up item.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - restore_mode -- Specify the restore mode.
    - vault_name -- Name of the Recovery services vault.

    Optional Parameters:
    - backup_management_type -- Specify the backup management type. Define how Azure Backup manages the backup of entities within the ARM resource. For eg: AzureWorkloads refers to workloads installed within Azure VMs, AzureStorage refers to entities within Storage account. Required only if friendly name is used as Container name.
    - filepath -- The path to which the DB should be restored as files.
    - from_full_rp_name -- Name of the starting Recovery point.
    - log_point_in_time -- Specify the point-in-time which will be restored.
    - rp_name -- Name of the recovery point.
    - target_container_name -- The target container to which the DB recovery point should be downloaded as files.
    - target_item_name -- Specify the target item name for the restore operation.
    - target_resource_group -- Specify the resource group of target item for Cross Region Restore. Default value will be same as --resource-group if not specified.
    - target_server_name -- Specify the parent server name of the target item.
    - target_server_type -- Specify the type of the server which should be discovered.
    - target_vault_name -- Specify the vault name of target item for Cross Region Restore. Default value will be same as --vault-name if not specified.
    - workload_type -- Specify the type of applications within the Resource which should be discovered and protected by Azure Backup. 
    '''
    return _call_az("az backup recoveryconfig show", locals())




from ... pyaz_utils import _call_az

def show(resource_group, service_name):
    '''
    

    Required Parameters:
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - service_name -- The name of the search service.
    '''
    return _call_az("az search admin-key show", locals())


def renew(key_kind, resource_group, service_name):
    '''
    

    Required Parameters:
    - key_kind -- The type (primary or secondary) of the admin key.
    - resource_group -- Name of resource group. You can configure the default group using `az configure --defaults group=<name>`
    - service_name -- The name of the search service.
    '''
    return _call_az("az search admin-key renew", locals())




'''
Manage admin information for certificate issuers.
'''
from ..... pyaz_utils import _call_az

def list(issuer_name, vault_name):
    '''
    

    Required Parameters:
    - issuer_name -- Certificate issuer name.
    - vault_name -- Name of the Vault.
    '''
    return _call_az("az keyvault certificate issuer admin list", locals())


def add(email, issuer_name, vault_name, first_name=None, last_name=None, phone=None):
    '''
    

    Required Parameters:
    - email -- Admin e-mail address. Must be unique within the vault.
    - issuer_name -- Certificate issuer name.
    - vault_name -- Name of the Vault.

    Optional Parameters:
    - first_name -- Admin first name.
    - last_name -- Admin last name.
    - phone -- Admin phone number.
    '''
    return _call_az("az keyvault certificate issuer admin add", locals())


def delete(email, issuer_name, vault_name):
    '''
    

    Required Parameters:
    - email -- Admin e-mail address. Must be unique within the vault.
    - issuer_name -- Certificate issuer name.
    - vault_name -- Name of the Vault.
    '''
    return _call_az("az keyvault certificate issuer admin delete", locals())

